@@ -1154,6 +1154,7 @@ public:
 	Firebird::IRequest*	handles_get_type_req_handle1;
 	Firebird::IRequest*	handles_get_user_privilege_req_handle1;
 	Firebird::IRequest*	handles_get_view_req_handle1;
+	Firebird::IRequest* handles_activateIndex_req_handle1;
 
 	// The handles_put.. are for backup.
 	Firebird::IRequest*	handles_put_index_req_handle1;
@@ -1226,6 +1227,9 @@ public:
 	bool gbl_stat_header;				// true, if stats header was printed
 	bool gbl_stat_done;					// true, if main process is done, stop to collect db-level stats
 	SINT64 gbl_stats[LAST_COUNTER];
+
+	bool gbl_use_no_auto_undo = true;
+	bool gbl_use_auto_release_temp_blobid = true;
 };
 
 // CVC: This aux routine declared here to not force inclusion of burp.h with burp_proto.h
