@@ -95,6 +95,7 @@ PARSER_TOKEN(TOK_BINARY, "BINARY", false)
 PARSER_TOKEN(TOK_BIND, "BIND", true)
 PARSER_TOKEN(TOK_BIT_LENGTH, "BIT_LENGTH", false)
 PARSER_TOKEN(TOK_BLOB, "BLOB", false)
+PARSER_TOKEN(TOK_BLOBID, "BLOBID", true)
 PARSER_TOKEN(TOK_BLOB_APPEND, "BLOB_APPEND", true)
 PARSER_TOKEN(TOK_BLOCK, "BLOCK", true)
 PARSER_TOKEN(TOK_BODY, "BODY", true)
@@ -486,6 +487,7 @@ PARSER_TOKEN(TOK_TAGS, "TAGS", true)
 PARSER_TOKEN(TOK_TAN, "TAN", true)
 PARSER_TOKEN(TOK_TANH, "TANH", true)
 PARSER_TOKEN(TOK_TARGET, "TARGET", true)
+PARSER_TOKEN(TOK_TEMP, "TEMP", true)
 PARSER_TOKEN(TOK_TEMPORARY, "TEMPORARY", true)
 PARSER_TOKEN(TOK_THEN, "THEN", false)
 PARSER_TOKEN(TOK_TIES, "TIES", true)
