@@ -577,6 +577,11 @@ public:
 	virtual void genBlr(DsqlCompilerScratch* dsqlScratch);
 	virtual void make(DsqlCompilerScratch* dsqlScratch, dsc* desc);
 
+	virtual bool possiblyUnknown() const
+	{
+		return true;
+	}
+
 	virtual bool ignoreNulls(const StreamList& /*streams*/) const
 	{
 		return false;
@@ -786,7 +791,7 @@ public:
 		dsqlDesc = desc;
 	}
 
-	virtual bool deterministic() const override
+	virtual bool deterministic() const
 	{
 		return true;
 	}
@@ -867,7 +872,7 @@ public:
 	virtual void genBlr(DsqlCompilerScratch* dsqlScratch);
 	virtual void make(DsqlCompilerScratch* dsqlScratch, dsc* desc);
 
-	virtual bool deterministic() const override
+	virtual bool deterministic() const
 	{
 		return false;
 	}
@@ -960,12 +965,6 @@ public:
 		return *reinterpret_cast<SLONG*>(litDesc.dsc_address);
 	}
 
-	const char* getText() const
-	{
-		fb_assert(litDesc.dsc_dtype == dtype_text);
-		return reinterpret_cast<const char*>(litDesc.dsc_address);
-	}
-
 	void fixMinSInt32(MemoryPool& pool);
 	void fixMinSInt64(MemoryPool& pool);
 	void fixMinSInt128(MemoryPool& pool);
@@ -1628,7 +1627,7 @@ public:
 
 	Request* getParamRequest(Request* request) const;
 
-	virtual bool deterministic() const override
+	virtual bool deterministic() const
 	{
 		return true;
 	}
@@ -1680,7 +1679,7 @@ public:
 	virtual void genBlr(DsqlCompilerScratch* dsqlScratch);
 	virtual void make(DsqlCompilerScratch* dsqlScratch, dsc* desc);
 
-	virtual bool deterministic() const override
+	virtual bool deterministic() const
 	{
 		return true;
 	}
@@ -2087,7 +2086,7 @@ public:
 	virtual void genBlr(DsqlCompilerScratch* dsqlScratch);
 	virtual void make(DsqlCompilerScratch* dsqlScratch, dsc* desc);
 
-	virtual bool deterministic() const override;
+	virtual bool deterministic() const;
 
 	virtual void getDesc(thread_db* tdbb, CompilerScratch* csb, dsc* desc);
 	virtual ValueExprNode* copy(thread_db* tdbb, NodeCopier& copier) const;
@@ -2169,7 +2168,7 @@ public:
 	virtual void genBlr(DsqlCompilerScratch* dsqlScratch);
 	virtual void make(DsqlCompilerScratch* dsqlScratch, dsc* desc);
 
-	virtual bool deterministic() const override;
+	virtual bool deterministic() const;
 
 	virtual bool possiblyUnknown() const
 	{
@@ -2269,7 +2268,7 @@ public:
 
 	Request* getVarRequest(Request* request) const;
 
-	virtual bool deterministic() const override
+	virtual bool deterministic() const
 	{
 		return false;
 	}
