@@ -1574,6 +1574,7 @@ public:
 		NODE_PRINT(printer, ignoreLimbo);
 		NODE_PRINT(printer, restartRequests);
 		NODE_PRINT(printer, autoCommit);
+		NODE_PRINT(printer, autoReleaseTempBlobID);
 		NODE_PRINT(printer, lockTimeout);
 		//// FIXME-PRINT: NODE_PRINT(printer, reserveList);
 		NODE_PRINT(printer, tpb);
@@ -1600,6 +1601,7 @@ public:
 	Nullable<bool> ignoreLimbo;
 	Nullable<bool> restartRequests;
 	Nullable<bool> autoCommit;
+	Nullable<bool> autoReleaseTempBlobID;
 };
 
 
