@@ -729,6 +729,7 @@ public:
 	void storeBinaryBlob(thread_db* tdbb, jrd_tra* transaction, bid* blobId,
 		const Firebird::ByteChunk& chunk);
 
+	void releaseBatches();
 	void releaseGTTs(thread_db* tdbb);
 	void resetSession(thread_db* tdbb, jrd_tra** traHandle);
 
@@ -791,12 +792,12 @@ public:
 	}
 
 	// batches control
-	void registerBatch(JBatch* b)
+	void registerBatch(DsqlBatch* b)
 	{
 		att_batches.add(b);
 	}
 
-	void deregisterBatch(JBatch* b)
+	void deregisterBatch(DsqlBatch* b)
 	{
 		att_batches.findAndRemove(b);
 	}
@@ -843,7 +844,7 @@ public:
 	ProfilerManager* getProfilerManager(thread_db* tdbb);
 	ProfilerManager* getActiveProfilerManagerForNonInternalStatement(thread_db* tdbb);
 	bool isProfilerActive();
-	void releaseProfilerManager();
+	void releaseProfilerManager(thread_db* tdbb);
 
 	JProvider* getProvider()
 	{
@@ -859,7 +860,7 @@ private:
 	unsigned int att_stmt_timeout;		// milliseconds
 	Firebird::RefPtr<Firebird::TimerImpl> att_idle_timer;
 
-	Firebird::Array<JBatch*> att_batches;
+	Firebird::Array<DsqlBatch*> att_batches;
 	InitialOptions att_initial_options;	// Initial session options
 	DebugOptions att_debug_options;
 	Firebird::AutoPtr<ProfilerManager> att_profiler_manager;	// ProfilerManager
