@@ -312,6 +312,9 @@ class Database : public pool_alloc<type_dbb>
 			return m_replConfig.get();
 		}
 
+		bool incTempCacheUsage(FB_SIZE_T size);
+		void decTempCacheUsage(FB_SIZE_T size);
+
 	private:
 		const Firebird::string m_id;
 		const Firebird::RefPtr<const Firebird::Config> m_config;
@@ -320,12 +323,16 @@ class Database : public pool_alloc<type_dbb>
 		Firebird::AutoPtr<EventManager> m_eventMgr;
 		Firebird::AutoPtr<Replication::Manager> m_replMgr;
 		Firebird::Mutex m_mutex;
+		std::atomic<FB_UINT64> m_tempCacheUsage;		// total size of in-memory temp space chunks (see TempSpace class)
+		const FB_UINT64 m_tempCacheLimit;
 
 		explicit GlobalObjectHolder(const Firebird::string& id,
 									const Firebird::PathName& filename,
 									Firebird::RefPtr<const Firebird::Config> config)
 			: m_id(getPool(), id), m_config(config),
-			  m_replConfig(Replication::Config::get(filename))
+			  m_replConfig(Replication::Config::get(filename)),
+			  m_tempCacheUsage(0),
+			  m_tempCacheLimit(m_config->getTempCacheLimit())
 		{}
 	};
 
@@ -445,8 +452,6 @@ public:
 
 	Lock*		dbb_retaining_lock;		// lock for preserving commit retaining snapshot
 	PageManager dbb_page_manager;
-	vcl*		dbb_t_pages;			// pages number for transactions
-	vcl*		dbb_gen_id_pages;		// known pages for gen_id
 	BlobFilter*	dbb_blob_filters;		// known blob filters
 
 	MonitoringData*			dbb_monitoring_data;	// monitoring data
@@ -457,6 +462,10 @@ private:
 	Firebird::SyncObject dbb_modules_sync;
 	DatabaseModules	dbb_modules;		// external function/filter modules
 
+	// Vectors of known pages and their synchronization
+	Firebird::SyncObject dbb_pages_sync;	// guard access to dbb_XXX_pages vectors
+	vcl* dbb_tip_pages;						// known TIP pages
+	vcl* dbb_gen_pages;						// known generator pages
 public:
 	Firebird::AutoPtr<ExtEngineManager>	dbb_extManager;	// external engine manager
 
@@ -489,9 +498,6 @@ public:
 	Firebird::SyncObject			dbb_sortbuf_sync;
 	Firebird::Array<UCHAR*>			dbb_sort_buffers;	// sort buffers ready for reuse
 
-	Firebird::Mutex dbb_temp_cache_mutex;
-	FB_UINT64 dbb_temp_cache_size;		// total size of in-memory temp space chunks (see TempSpace class)
-
 	TraNumber dbb_oldest_active;		// Cached "oldest active" transaction
 	TraNumber dbb_oldest_transaction;	// Cached "oldest interesting" transaction
 	TraNumber dbb_oldest_snapshot;		// Cached "oldest snapshot" of all active transactions
@@ -547,6 +553,9 @@ public:
 	// returns an unique ID string for a database file
 	const Firebird::string& getUniqueFileId();
 
+	// returns the minimum IO block size
+	ULONG getIOBlockSize() const;
+
 #ifdef DEV_BUILD
 	// returns true if main lock is in exclusive state
 	bool locked() const
@@ -583,6 +592,12 @@ public:
 		return ENCODE_ODS(dbb_ods_version, dbb_minor_version);
 	}
 
+	// Methods encapsulating operations with vectors of known pages
+	ULONG getKnownPagesCount(SCHAR ptype);
+	ULONG getKnownPage(SCHAR ptype, ULONG sequence);
+	void setKnownPage(SCHAR ptype, ULONG sequence, ULONG value);
+	void copyKnownPages(SCHAR ptype, ULONG count, ULONG* data);
+
 private:
 	Database(MemoryPool* p, Firebird::IPluginConfig* pConf, bool shared)
 	:	dbb_permanent(p),
@@ -661,6 +676,7 @@ public:
 	static int replStateAst(void*);
 
 	const CoercionArray *getBindings() const;
+	void startTipCache(thread_db* tdbb);
 
 	void initGlobalObjects();
 	void shutdownGlobalObjects();
@@ -685,6 +701,16 @@ public:
 		return dbb_gblobj_holder->getReplConfig();
 	}
 
+	bool incTempCacheUsage(FB_SIZE_T size)
+	{
+		return dbb_gblobj_holder->incTempCacheUsage(size);
+	}
+
+	void decTempCacheUsage(FB_SIZE_T size)
+	{
+		dbb_gblobj_holder->decTempCacheUsage(size);
+	}
+
 private:
 	//static int blockingAstSharedCounter(void*);
 	static int blocking_ast_sweep(void* ast_object);
