@@ -646,7 +646,7 @@ void EXE_receive(thread_db* tdbb,
 
 		// ASF: temporary blobs returned to the client should not be released
 		// with the request, but in the transaction end.
-		if (top_level)
+		if (top_level || transaction->tra_temp_blobs_count)
 		{
 			for (int i = 0; i < format->fmt_count; ++i)
 			{
@@ -660,7 +660,8 @@ void EXE_receive(thread_db* tdbb,
 					{
 						BlobIndex* current = &transaction->tra_blobs->current();
 
-						if (current->bli_request &&
+						if (top_level &&
+							current->bli_request &&
 							current->bli_request->req_blobs.locate(id->bid_temp_id()))
 						{
 							current->bli_request->req_blobs.fastRemove();
@@ -668,12 +669,13 @@ void EXE_receive(thread_db* tdbb,
 						}
 
 						if (!current->bli_materialized &&
-							(current->bli_blob_object->blb_flags & BLB_close_on_read))
+							(current->bli_blob_object->blb_flags & (BLB_close_on_read | BLB_stream)) ==
+								(BLB_close_on_read | BLB_stream))
 						{
 							current->bli_blob_object->BLB_close(tdbb);
 						}
 					}
-					else
+					else if (top_level)
 					{
 						transaction->checkBlob(tdbb, id, NULL, false);
 					}
@@ -750,10 +752,15 @@ void EXE_release(thread_db* tdbb, Request* request)
 		if (request->req_attachment->att_requests.find(request, pos))
 			request->req_attachment->att_requests.remove(pos);
 
-		request->req_attachment = NULL;
+		request->req_attachment = nullptr;
 	}
 
 	request->req_flags &= ~req_in_use;
+	if (request->req_timer)
+	{
+		request->req_timer->stop();
+		request->req_timer = nullptr;
+	}
 }
 
 
@@ -1110,7 +1117,7 @@ void EXE_execute_triggers(thread_db* tdbb,
  *	if any blow up.
  *
  **************************************/
-	if (!*triggers)
+	if (!*triggers || (*triggers)->isEmpty())
 		return;
 
 	SET_TDBB(tdbb);
@@ -1118,7 +1125,7 @@ void EXE_execute_triggers(thread_db* tdbb,
 	Request* const request = tdbb->getRequest();
 	jrd_tra* const transaction = request ? request->req_transaction : tdbb->getTransaction();
 
-	TrigVector* vector = *triggers;
+	RefPtr<TrigVector> vector(*triggers);
 	Record* const old_rec = old_rpb ? old_rpb->rpb_record : NULL;
 	Record* const new_rec = new_rpb ? new_rpb->rpb_record : NULL;
 
@@ -1243,15 +1250,9 @@ void EXE_execute_triggers(thread_db* tdbb,
 
 			trigger = NULL;
 		}
-
-		if (vector != *triggers)
-			MET_release_triggers(tdbb, &vector, true);
 	}
 	catch (const Exception& ex)
 	{
-		if (vector != *triggers)
-			MET_release_triggers(tdbb, &vector, true);
-
 		if (trigger)
 		{
 			EXE_unwind(tdbb, trigger);
