@@ -115,6 +115,9 @@ Manager::~Manager()
 
 void Manager::addProvider(Provider* provider)
 {
+	// TODO: if\when usage of user providers will be implemented,
+	// need to check provider name for allowed chars (file system rules ?)
+
 	for (const Provider* prv = m_providers; prv; prv = prv->m_next)
 	{
 		if (prv->m_name == provider->m_name) {
@@ -155,6 +158,15 @@ static void splitDataSourceName(thread_db* tdbb, const string& dataSource,
 	else
 	{
 		FB_SIZE_T pos = dataSource.find("::");
+
+		// Check if it is part of IPv6 address, assume provider name can't contain square brackets
+		if (pos != string::npos &&
+			dataSource.rfind("[", pos) != string::npos &&
+			dataSource.find("]", pos) != string::npos)
+		{
+			pos = string::npos;
+		}
+
 		if (pos != string::npos)
 		{
 			prvName = dataSource.substr(0, pos);
@@ -195,15 +207,15 @@ Connection* Manager::getConnection(thread_db* tdbb, const string& dataSource,
 
 	Provider* prv = getProvider(prvName);
 
-	const bool isCurrent = (prvName == INTERNAL_PROVIDER_NAME) &&
+	const bool isCurrentAtt = (prvName == INTERNAL_PROVIDER_NAME) &&
 		isCurrentAccount(att->att_user, user, pwd, role);
 
 	ClumpletWriter dpb(ClumpletReader::dpbList, MAX_DPB_SIZE);
-	if (!isCurrent)
+	if (!isCurrentAtt)
 		prv->generateDPB(tdbb, dpb, user, pwd, role);
 
 	// look up at connections already bound to current attachment
-	Connection* conn = prv->getBoundConnection(tdbb, dbName, dpb, tra_scope);
+	Connection* conn = prv->getBoundConnection(tdbb, dbName, dpb, tra_scope, isCurrentAtt);
 	if (conn)
 		return conn;
 
@@ -214,20 +226,23 @@ Connection* Manager::getConnection(thread_db* tdbb, const string& dataSource,
 
 	ULONG hash = 0;
 
-	if (!isCurrent)
+	if (!isCurrentAtt)
 	{
+		CryptHash ch(att->att_crypt_callback);
 		hash = DefaultHash<UCHAR>::hash(dbName.c_str(), dbName.length(), MAX_ULONG) +
-			   DefaultHash<UCHAR>::hash(dpb.getBuffer(), dpb.getBufferLength(), MAX_ULONG);
+			   DefaultHash<UCHAR>::hash(dpb.getBuffer(), dpb.getBufferLength(), MAX_ULONG) +
+			   DefaultHash<UCHAR>::hash(ch.getValue(), ch.getLength(), MAX_ULONG);
 
 		while (true)
 		{
-			conn = m_connPool->getConnection(tdbb, prv, hash, dbName, dpb);
+			conn = m_connPool->getConnection(tdbb, prv, hash, dbName, dpb, ch);
 			if (!conn)
 				break;
 
 			if (conn->validate(tdbb))
 			{
 				prv->bindConnection(tdbb, conn);
+				conn->resetRedirect(att->att_crypt_callback);
 				break;
 			}
 
@@ -240,7 +255,7 @@ Connection* Manager::getConnection(thread_db* tdbb, const string& dataSource,
 	{
 		// finally, create new connection
 		conn = prv->createConnection(tdbb, dbName, dpb, tra_scope);
-		if (!isCurrent)
+		if (!isCurrentAtt)
 			m_connPool->addConnection(tdbb, conn, hash);
 	}
 
@@ -344,6 +359,9 @@ Connection* Provider::createConnection(thread_db* tdbb,
 {
 	Connection* conn = doCreateConnection();
 	conn->setup(dbName, dpb);
+	if (!conn->isCurrent())
+		conn->setCallbackRedirect(tdbb->getAttachment()->att_crypt_callback);
+
 	try
 	{
 		conn->attach(tdbb);
@@ -375,10 +393,13 @@ void Provider::bindConnection(thread_db* tdbb, Connection* conn)
 
 Connection* Provider::getBoundConnection(Jrd::thread_db* tdbb,
 	const Firebird::PathName& dbName, Firebird::ClumpletReader& dpb,
-	TraScope tra_scope)
+	TraScope tra_scope, bool isCurrentAtt)
 {
 	Database* dbb = tdbb->getDatabase();
 	Attachment* att = tdbb->getAttachment();
+	CryptHash ch;
+	if (!isCurrentAtt)
+		ch.assign(att->att_crypt_callback);
 
 	MutexLockGuard guard(m_mutex, FB_FUNCTION);
 
@@ -392,7 +413,7 @@ Connection* Provider::getBoundConnection(Jrd::thread_db* tdbb,
 			if (conn->getBoundAtt() != att)
 				break;
 
-			if (conn->isSameDatabase(dbName, dpb) &&
+			if (conn->isSameDatabase(dbName, dpb, ch) &&
 				conn->isAvailable(tdbb, tra_scope))
 			{
 				fb_assert(conn->getProvider() == this);
@@ -402,6 +423,7 @@ Connection* Provider::getBoundConnection(Jrd::thread_db* tdbb,
 					continue;
 #endif
 
+				conn->resetRedirect(att->att_crypt_callback);
 				return conn;
 			}
 		} while (acc.getNext());
@@ -441,9 +463,9 @@ void Provider::jrdAttachmentEnd(thread_db* tdbb, Jrd::Attachment* att, bool forc
 void Provider::releaseConnection(thread_db* tdbb, Connection& conn, bool inPool)
 {
 	ConnectionsPool* connPool = conn.getConnPool();
+	Attachment* att = conn.getBoundAtt();
 
 	{ // m_mutex scope
-		Attachment* att = conn.getBoundAtt();
 		MutexLockGuard guard(m_mutex, FB_FUNCTION);
 
 		bool found = false;
@@ -464,7 +486,43 @@ void Provider::releaseConnection(thread_db* tdbb, Connection& conn, bool inPool)
 			m_connections.add(AttToConn(NULL, &conn));
 	}
 
-	if (!inPool || !connPool || !conn.isConnected() || !conn.resetSession(tdbb))
+	FbLocalStatus resetError;
+	inPool = inPool && connPool && connPool->getMaxCount() &&
+		conn.isConnected() && conn.hasValidCryptCallback();
+
+	if (inPool)
+	{
+		inPool = conn.resetSession(tdbb);
+
+		// Check if reset of external session failed when parent (local) attachment
+		// is resetting or run ON DISCONNECT trigger.
+
+		const auto status = tdbb->tdbb_status_vector;
+		if (!inPool && status->hasData())
+		{
+			if (att->att_flags & ATT_resetting)
+				resetError.loadFrom(status);
+			else
+			{
+				auto req = tdbb->getRequest();
+				while (req)
+				{
+					if (req->req_trigger_action == TRIGGER_DISCONNECT)
+					{
+						resetError.loadFrom(status);
+						break;
+					}
+					req = req->req_caller;
+				}
+			}
+		}
+	}
+
+	if (inPool)
+	{
+		connPool->putConnection(tdbb, &conn);
+	}
+	else
 	{
 		{	// scope
 			MutexLockGuard guard(m_mutex, FB_FUNCTION);
@@ -477,9 +535,8 @@ void Provider::releaseConnection(thread_db* tdbb, Connection& conn, bool inPool)
 			connPool->delConnection(tdbb, &conn, false);
 
 		Connection::deleteConnection(tdbb, &conn);
+		resetError.check();
 	}
-	else
-		connPool->putConnection(tdbb, &conn);
 }
 
 void Provider::clearConnections(thread_db* tdbb)
@@ -561,7 +618,7 @@ Connection::~Connection()
 	fb_assert(m_boundAtt == NULL);
 }
 
-bool Connection::isSameDatabase(const PathName& dbName, ClumpletReader& dpb) const
+bool Connection::isSameDatabase(const PathName& dbName, ClumpletReader& dpb, const CryptHash& hash) const
 {
 	if (m_dbName != dbName)
 		return false;
@@ -570,7 +627,8 @@ bool Connection::isSameDatabase(const PathName& dbName, ClumpletReader& dpb) con
 	// but in different order
 
 	const FB_SIZE_T len = m_dpb.getCount();
-	return (len == dpb.getBufferLength()) && (memcmp(m_dpb.begin(), dpb.getBuffer(), len) == 0);
+	return (len == dpb.getBufferLength()) && (memcmp(m_dpb.begin(), dpb.getBuffer(), len) == 0) &&
+		(m_cryptCallbackRedir == hash);
 }
 
 
@@ -870,7 +928,7 @@ ConnectionsPool::Data* ConnectionsPool::removeOldest()
 }
 
 Connection* ConnectionsPool::getConnection(thread_db* tdbb, Provider* prv, ULONG hash,
-	const PathName& dbName, ClumpletReader& dpb)
+	const PathName& dbName, ClumpletReader& dpb, const CryptHash& ch)
 {
 	MutexLockGuard guard(m_mutex, FB_FUNCTION);
 
@@ -886,7 +944,7 @@ Connection* ConnectionsPool::getConnection(thread_db* tdbb, Provider* prv, ULONG
 			break;
 
 		Connection* conn = item->m_conn;
-		if (conn->getProvider() == prv && conn->isSameDatabase(dbName, dpb))
+		if (conn->getProvider() == prv && conn->isSameDatabase(dbName, dpb, ch))
 		{
 			m_idleArray.remove(pos);
 			removeFromList(&m_idleList, item);
@@ -905,7 +963,8 @@ void ConnectionsPool::putConnection(thread_db* tdbb, Connection* conn)
 	fb_assert(conn->getConnPool() == this);
 
 	Connection* oldConn = NULL;
-	bool startIdleTimer = false;
+	Firebird::RefPtr<IdleTimer>	timer;
+
 	if (m_maxCount > 0)
 	{
 		MutexLockGuard guard(m_mutex, FB_FUNCTION);
@@ -977,7 +1036,7 @@ void ConnectionsPool::putConnection(thread_db* tdbb, Connection* conn)
 			if (!m_timer)
 				m_timer = FB_NEW IdleTimer(*this);
 
-			startIdleTimer = true;
+			timer = m_timer;
 		}
 
 #ifdef EDS_DEBUG
@@ -996,8 +1055,10 @@ void ConnectionsPool::putConnection(thread_db* tdbb, Connection* conn)
 	if (oldConn)
 		oldConn->getProvider()->releaseConnection(tdbb, *oldConn, false);
 
-	if (startIdleTimer)
-		m_timer->start();
+	// Note, m_timer could be cleared at this point - due to shutdown.
+	// Then m_idleList will be empty and start() will do nothing.
+	if (timer)
+		timer->start();
 }
 
 void ConnectionsPool::addConnection(thread_db* tdbb, Connection* conn, ULONG hash)
@@ -2582,4 +2643,62 @@ EngineCallbackGuard::~EngineCallbackGuard()
 	}
 }
 
+
+// CryptHash
+
+CryptHash::CryptHash(ICryptKeyCallback* callback)
+	: m_value(*getDefaultMemoryPool())
+{
+	assign(callback);
+}
+
+CryptHash::CryptHash()
+	: m_value(*getDefaultMemoryPool())
+{ }
+
+void CryptHash::assign(ICryptKeyCallback* callback)
+{
+	fb_assert(!isValid());
+
+	FbLocalStatus status;
+
+	int len = callback->getHashLength(&status);
+	if (len > 0 && status.isSuccess())
+		callback->getHashData(&status, m_value.getBuffer(len));
+
+	m_valid = (len >= 0) && status.isSuccess();
+}
+
+bool CryptHash::operator==(const CryptHash& h) const
+{
+	return (isValid() == h.isValid()) && (m_value == h.m_value);
+}
+
+
+// CryptCallbackRedirector
+
+void CryptCallbackRedirector::setRedirect(Firebird::ICryptKeyCallback* originalCallback)
+{
+	m_hash.assign(originalCallback);
+
+	if (m_hash.isValid())
+		m_keyCallback = originalCallback;
+}
+
+void CryptCallbackRedirector::resetRedirect(Firebird::ICryptKeyCallback* newCallback)
+{
+#ifdef DEV_BUILD
+	CryptHash ch(newCallback);
+	fb_assert(isValid() && ch.isValid());
+	fb_assert(m_hash == ch);
+#endif
+
+	m_keyCallback = newCallback;
+}
+
+bool CryptCallbackRedirector::operator==(const CryptHash& ch) const
+{
+	return m_hash == ch;
+}
+
 } // namespace EDS
