@@ -899,16 +899,18 @@ void BackupManager::setForcedWrites(const bool forceWrite, const bool notUseFSCa
 
 BackupManager::BackupManager(thread_db* tdbb, Database* _database, int ini_state) :
 	dbCreating(false), database(_database), diff_file(NULL), alloc_table(NULL),
-	last_allocated_page(0), current_scn(0), diff_name(*_database->dbb_permanent),
+	last_allocated_page(0), temp_buffers_space(*database->dbb_permanent),
+	current_scn(0), diff_name(*database->dbb_permanent),
 	explicit_diff_name(false), flushInProgress(false), shutDown(false), allocIsValid(false),
 	master(false), stateBlocking(false),
 	stateLock(FB_NEW_POOL(*database->dbb_permanent) NBackupStateLock(tdbb, *database->dbb_permanent, this)),
 	allocLock(FB_NEW_POOL(*database->dbb_permanent) NBackupAllocLock(tdbb, *database->dbb_permanent, this))
 {
 	// Allocate various database page buffers needed for operation
-	temp_buffers_space = FB_NEW_POOL(*database->dbb_permanent) BYTE[database->dbb_page_size * 3 + PAGE_ALIGNMENT];
 	// Align it at sector boundary for faster IO (also guarantees correct alignment for ULONG later)
-	BYTE* temp_buffers = reinterpret_cast<BYTE*>(FB_ALIGN(temp_buffers_space, PAGE_ALIGNMENT));
+	UCHAR* temp_buffers = reinterpret_cast<UCHAR*>
+		(temp_buffers_space.getAlignedBuffer(database->dbb_page_size * 3, database->getIOBlockSize()));
+
 	memset(temp_buffers, 0, database->dbb_page_size * 3);
 
 	backup_state = ini_state;
@@ -925,7 +927,6 @@ BackupManager::~BackupManager()
 	delete stateLock;
 	delete allocLock;
 	delete alloc_table;
-	delete[] temp_buffers_space;
 }
 
 void BackupManager::setDifference(thread_db* tdbb, const char* filename)
