@@ -108,6 +108,8 @@ void InnerJoin::calculateStreamInfo()
 		innerStream->baseIndexes = candidate->indexes;
 		innerStream->baseUnique = candidate->unique;
 		innerStream->baseNavigated = candidate->navigated;
+		innerStream->baseMatches = candidate->matches;
+		innerStream->baseDependentFromStreams = candidate->dependentFromStreams;
 
 		csb->csb_rpt[innerStream->number].deactivate();
 	}
@@ -191,6 +193,12 @@ void InnerJoin::estimateCost(unsigned position,
 	// Get the stream cardinality
 	const auto streamCardinality = csb->csb_rpt[stream->number].csb_cardinality;
 
+	// If the table looks like empty during preparation time, we cannot be sure about
+	// its real cardinality during execution. So, unless we have some index-based
+	// filtering applied, let's better be pessimistic and avoid hash joining due to
+	// likely cardinality under-estimation.
+	const bool avoidHashJoin = (streamCardinality <= MINIMUM_CARDINALITY && !stream->baseIndexes);
+
 	auto currentCardinality = candidate->unique ?
 		MINIMUM_CARDINALITY : streamCardinality * candidate->selectivity;
 	auto currentCost = candidate->cost;
@@ -210,7 +218,7 @@ void InnerJoin::estimateCost(unsigned position,
 	// Consider whether the current stream can be hash-joined to the prior ones.
 	// Beware conditional retrievals, this is impossible for them.
 
-	if (position && !candidate->condition)
+	if (position && !candidate->condition && !avoidHashJoin)
 	{
 		// Calculate the hashing cost. It consists of the following parts:
 		//  - hashed stream retrieval
@@ -232,6 +240,13 @@ void InnerJoin::estimateCost(unsigned position,
 			// Scan the matches for possible equi-join conditions
 			for (const auto match : candidate->matches)
 			{
+				if (!match->containsStream(stream->number))
+				{
+					// This should never happen but be prepared for the worst
+					fb_assert(false);
+					continue;
+				}
+
 				// Check whether we have an equivalence operation
 				if (!optimizer->checkEquiJoin(match))
 					continue;
@@ -566,13 +581,15 @@ River* InnerJoin::formRiver()
 
 			// Create a hash join
 			rsb = FB_NEW_POOL(getPool())
-				HashJoin(tdbb, csb, 2, hashJoinRsbs, keys.begin(), stream.selectivity);
+				HashJoin(tdbb, csb, INNER_JOIN, 2, hashJoinRsbs, keys.begin(), stream.selectivity);
 
 			// Clear priorly processed rsb's, as they're already incorporated into a hash join
 			rsbs.clear();
 		}
 		else
+		{
 			rsb = optimizer->generateRetrieval(stream.number, sortPtr, false, false);
+		}
 
 		rsbs.add(rsb);
 		streams.add(stream.number);
