@@ -168,12 +168,18 @@ namespace
 	class CrossJoin : public River
 	{
 	public:
-		CrossJoin(CompilerScratch* csb, RiverList& rivers)
-			: River(csb, nullptr, rivers)
+		CrossJoin(Optimizer* opt, RiverList& rivers, JoinType joinType)
+			: River(opt->getCompilerScratch(), nullptr, rivers)
 		{
+			fb_assert(joinType != OUTER_JOIN);
+
+			const auto csb = opt->getCompilerScratch();
+			Optimizer::ConjunctIterator iter(opt->getBaseConjuncts());
+
 			// Save states of the underlying streams and restore them afterwards
 
 			StreamStateHolder stateHolder(csb, m_streams);
+			stateHolder.deactivate();
 
 			// Generate record source objects
 
@@ -181,57 +187,80 @@ namespace
 
 			if (riverCount == 1)
 			{
-				River* const sub_river = rivers.pop();
-				m_rsb = sub_river->getRecordSource();
+				const auto subRiver = rivers.pop();
+				const auto subRsb = subRiver->getRecordSource();
+				subRiver->activate(csb);
+				m_rsb = opt->applyBoolean(subRsb, iter);
 			}
 			else
 			{
 				HalfStaticArray<RecordSource*, OPT_STATIC_ITEMS> rsbs(riverCount);
 
-				// Reorder input rivers according to their possible inter-dependencies
-
-				while (rivers.hasData())
+				if (joinType == INNER_JOIN)
 				{
-					const auto orgCount = rsbs.getCount();
+					// Reorder input rivers according to their possible inter-dependencies
 
-					for (auto& subRiver : rivers)
+					while (rivers.hasData())
 					{
-						const auto subRsb = subRiver->getRecordSource();
-						fb_assert(!rsbs.exist(subRsb));
+						const auto orgCount = rsbs.getCount();
 
-						subRiver->activate(csb);
-
-						if (subRiver->isComputable(csb))
+						for (auto& subRiver : rivers)
 						{
-							rsbs.add(subRsb);
-							rivers.remove(&subRiver);
-							break;
+							auto subRsb = subRiver->getRecordSource();
+
+							subRiver->activate(csb);
+							subRsb = opt->applyBoolean(subRsb, iter);
+
+							if (subRiver->isComputable(csb))
+							{
+								rsbs.add(subRsb);
+								rivers.remove(&subRiver);
+								break;
+							}
+
+							subRiver->deactivate(csb);
 						}
 
-						subRiver->deactivate(csb);
+						if (rsbs.getCount() == orgCount)
+							break;
 					}
 
-					if (rsbs.getCount() == orgCount)
-						break;
-				}
-
-				if (rivers.hasData())
-				{
-					// Ideally, we should never get here. But just in case it happened, handle it.
-
-					for (auto& subRiver : rivers)
+					if (rivers.hasData())
 					{
-						const auto subRsb = subRiver->getRecordSource();
-						fb_assert(!rsbs.exist(subRsb));
+						// Ideally, we should never get here. But just in case it happened, handle it.
 
-						const auto pos = &subRiver - rivers.begin();
-						rsbs.insert(pos, subRsb);
+						for (auto& subRiver : rivers)
+						{
+							auto subRsb = subRiver->getRecordSource();
+
+							subRiver->activate(csb);
+							subRsb = opt->applyBoolean(subRsb, iter);
+
+							const auto pos = &subRiver - rivers.begin();
+							rsbs.insert(pos, subRsb);
+						}
+
+						rivers.clear();
+					}
+				}
+				else
+				{
+					for (const auto subRiver : rivers)
+					{
+						auto subRsb = subRiver->getRecordSource();
+
+						subRiver->activate(csb);
+						if (subRiver != rivers.front())
+							subRsb = opt->applyBoolean(subRsb, iter);
+
+						rsbs.add(subRsb);
 					}
 
 					rivers.clear();
 				}
 
-				m_rsb = FB_NEW_POOL(csb->csb_pool) NestedLoopJoin(csb, rsbs.getCount(), rsbs.begin());
+				m_rsb = FB_NEW_POOL(csb->csb_pool)
+					NestedLoopJoin(csb, rsbs.getCount(), rsbs.begin(), joinType);
 			}
 		}
 	};
@@ -266,7 +295,6 @@ namespace
 		}
 	}
 
-
 	unsigned getRiverCount(unsigned count, const ValueExprNode* const* eq_class)
 	{
 		// Given an sort/merge join equivalence class (vector of node pointers
@@ -569,7 +597,6 @@ Optimizer::Optimizer(thread_db* aTdbb, CompilerScratch* aCsb, RseNode* aRse, boo
 	  compileStreams(getPool()),
 	  bedStreams(getPool()),
 	  keyStreams(getPool()),
-	  subStreams(getPool()),
 	  outerStreams(getPool()),
 	  conjuncts(getPool())
 {
@@ -672,7 +699,7 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 	// AB: If we have limit our retrieval with FIRST / SKIP syntax then
 	// we may not deliver above conditions (from higher rse's) to this
 	// rse, because the results should be consistent.
-	if (rse->rse_skip || rse->rse_first)
+	if (rse->rse_skip || rse->rse_first || isSemiJoined())
 		parentStack = nullptr;
 
 	// Set base-point before the parent/distributed nodes begin.
@@ -814,14 +841,20 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 	// Go through the record selection expression generating
 	// record source blocks for all streams
 
-	RiverList rivers;
+	RiverList rivers, dependentRivers, specialRivers;
 
 	bool innerSubStream = false;
+
 	for (auto node : rse->rse_relations)
 	{
 		fb_assert(sort == rse->rse_sorted);
 		fb_assert(aggregate == rse->rse_aggregate);
 
+		const auto subRse = nodeAs<RseNode>(node);
+
+		const bool semiJoin = (subRse && subRse->isSemiJoined());
+		fb_assert(!semiJoin || rse->rse_jointype == blr_inner);
+
 		// Find the stream number and place it at the end of the bedStreams array
 		// (if this is really a stream and not another RseNode)
 
@@ -840,11 +873,13 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 			StreamList localStreams;
 			rsb->findUsedStreams(localStreams);
 
+			bool computable = false;
+
 			// AB: Save all outer-part streams
 			if (isInnerJoin() || (isLeftJoin() && !innerSubStream))
 			{
-				subStreams.join(localStreams);
-				outerStreams.join(localStreams);
+				if (node->computable(csb, INVALID_STREAM, false))
+					computable = true;
 
 				// Apply local booleans, if any. Note that it's done
 				// only for inner joins and outer streams of left joins.
@@ -854,10 +889,24 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 
 			const auto river = FB_NEW_POOL(getPool()) River(csb, rsb, node, localStreams);
 			river->deactivate(csb);
-			rivers.add(river);
+
+			if (computable)
+			{
+				outerStreams.join(localStreams);
+
+				if (semiJoin)
+					specialRivers.add(river);
+				else
+					rivers.add(river);
+			}
+			else
+			{
+				dependentRivers.add(river);
+			}
 		}
 		else
 		{
+			fb_assert(!semiJoin);
 			// We have a relation, just add its stream
 			fb_assert(bedStreams.hasData());
 			outerStreams.add(bedStreams.back());
@@ -872,20 +921,15 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 	if (compileStreams.getCount() > 5)
 		CCH_expand(tdbb, (ULONG) (compileStreams.getCount() * CACHE_PAGES_PER_STREAM));
 
-	// At this point we are ready to start optimizing.
-	// We will use the opt block to hold information of
-	// a global nature, meaning that it needs to stick
-	// around for the rest of the optimization process.
-
 	// Attempt to optimize aggregates via an index, if possible
 	if (aggregate && !sort)
 		sort = aggregate;
 	else
 		rse->rse_aggregate = aggregate = nullptr;
 
-	// AB: Mark the previous used streams (sub-RseNode's) as active
-	for (const auto subStream : subStreams)
-		csb->csb_rpt[subStream].activate();
+	// Activate the priorly used rivers
+	for (const auto river : rivers)
+		river->activate(csb);
 
 	bool sortCanBeUsed = true;
 	SortNode* const orgSortNode = sort;
@@ -904,9 +948,14 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 
 	// Outer joins are processed their own way
 	if (!isInnerJoin())
+	{
+		rivers.join(dependentRivers);
 		rsb = generateOuterJoin(rivers, &sort);
+	}
 	else
 	{
+		JoinType joinType = INNER_JOIN;
+
 		// AB: If previous rsb's are already on the stack we can't use
 		// a navigational-retrieval for an ORDER BY because the next
 		// streams are JOINed to the previous ones
@@ -917,7 +966,7 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 
 			// AB: We could already have multiple rivers at this
 			// point so try to do some hashing or sort/merging now.
-			while (generateEquiJoin(rivers))
+			while (generateEquiJoin(rivers, joinType))
 				;
 		}
 
@@ -954,7 +1003,7 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 				// Generate one river which holds a cross join rsb between
 				// all currently available rivers
 
-				rivers.add(FB_NEW_POOL(getPool()) CrossJoin(csb, rivers));
+				rivers.add(FB_NEW_POOL(getPool()) CrossJoin(this, rivers, joinType));
 				rivers.back()->activate(csb);
 			}
 			else
@@ -971,14 +1020,51 @@ RecordSource* Optimizer::compile(BoolExprNodeStack* parentStack)
 			}
 		}
 
-		// attempt to form joins in decreasing order of desirability
+		// Attempt to form joins in decreasing order of desirability
 		generateInnerJoin(joinStreams, rivers, &sort, rse->rse_plan);
 
-		// If there are multiple rivers, try some hashing or sort/merging
-		while (generateEquiJoin(rivers))
-			;
+		if (rivers.isEmpty() && dependentRivers.isEmpty())
+		{
+			// This case may look weird, but it's possible for recursive unions
+			rsb = FB_NEW_POOL(csb->csb_pool) NestedLoopJoin(csb, 0, nullptr, joinType);
+		}
+		else
+		{
+			while (rivers.hasData() || dependentRivers.hasData())
+			{
+				// Re-activate remaining rivers to be hashable/mergeable
+				for (const auto river : rivers)
+					river->activate(csb);
 
-		rsb = CrossJoin(csb, rivers).getRecordSource();
+				// If there are multiple rivers, try some hashing or sort/merging
+				while (generateEquiJoin(rivers, joinType))
+					;
+
+				if (dependentRivers.hasData())
+				{
+					fb_assert(joinType == INNER_JOIN);
+
+					rivers.join(dependentRivers);
+					dependentRivers.clear();
+				}
+
+				const auto finalRiver = FB_NEW_POOL(getPool()) CrossJoin(this, rivers, joinType);
+				fb_assert(rivers.isEmpty());
+				rsb = finalRiver->getRecordSource();
+
+				if (specialRivers.hasData())
+				{
+					fb_assert(joinType == INNER_JOIN);
+					joinType = SEMI_JOIN;
+
+					rivers.add(finalRiver);
+					rivers.join(specialRivers);
+					specialRivers.clear();
+				}
+			}
+		}
+
+		fb_assert(rsb);
 
 		// Pick up any residual boolean that may have fallen thru the cracks
 		rsb = generateResidualBoolean(rsb);
@@ -1603,6 +1689,52 @@ SortedStream* Optimizer::generateSort(const StreamList& streams,
 }
 
 
+//
+// Compose a filter including all computable booleans
+//
+
+RecordSource* Optimizer::applyBoolean(RecordSource* rsb,
+									  ConjunctIterator& iter)
+{
+	BoolExprNode* boolean = nullptr;
+	double selectivity = MAXIMUM_SELECTIVITY;
+
+	for (iter.rewind(); iter.hasData(); ++iter)
+	{
+		if (!(iter & CONJUNCT_USED) &&
+			!(iter->nodFlags & ExprNode::FLAG_RESIDUAL) &&
+			iter->computable(csb, INVALID_STREAM, false))
+		{
+			compose(getPool(), &boolean, iter);
+			iter |= CONJUNCT_USED;
+
+			if (!(iter & (CONJUNCT_MATCHED | CONJUNCT_JOINED)))
+				selectivity *= getSelectivity(*iter);
+		}
+	}
+
+	return boolean ? FB_NEW_POOL(getPool()) FilteredStream(csb, rsb, boolean, selectivity) : rsb;
+}
+
+
+//
+// Find conjuncts local to the given river and compose an appropriate filter
+//
+
+RecordSource* Optimizer::applyLocalBoolean(RecordSource* rsb,
+										   const StreamList& streams,
+										   ConjunctIterator& iter)
+{
+	StreamStateHolder globalHolder(csb);
+	globalHolder.deactivate();
+
+	StreamStateHolder localHolder(csb, streams);
+	localHolder.activate();
+
+	return applyBoolean(rsb, iter);
+}
+
+
 //
 // Check to make sure that the user-specified indices were actually utilized by the optimizer
 //
@@ -1927,14 +2059,15 @@ void Optimizer::checkSorts()
 
 unsigned Optimizer::distributeEqualities(BoolExprNodeStack& orgStack, unsigned baseCount)
 {
-	// dimitr:	Dumb protection against too many injected conjuncts (see CORE-5381).
-	//			Don't produce more additional conjuncts than we originally had
-	//			(i.e. this routine should never more than double the number of conjuncts).
-	//			Ideally, we need two separate limits here:
-	//				1) number of injected conjuncts (affects required impure size)
-	//				2) number of input conjuncts (affects search time inside this routine)
+	// dimitr:	Simplified protection against too many injected conjuncts (see CORE-5381).
+	//			Two separate limits are applied here:
+	//				1) number of input conjuncts (affects search time inside this routine)
+	//				2) number of injected conjuncts (affects required impure size)
 
-	if (baseCount * 2 > MAX_CONJUNCTS)
+	constexpr unsigned MAX_CONJUNCTS_TO_PROCESS = 1024;
+	const unsigned MAX_CONJUNCTS_TO_INJECT = MAX(baseCount, 256);
+
+	if (baseCount > MAX_CONJUNCTS_TO_PROCESS)
 		return 0;
 
 	ObjectsArray<ValueExprNodeStack> classes;
@@ -2019,7 +2152,7 @@ unsigned Optimizer::distributeEqualities(BoolExprNodeStack& orgStack, unsigned b
 			{
 				for (ValueExprNodeStack::iterator inner(outer); (++inner).hasData(); )
 				{
-					if (count < baseCount)
+					if (count < MAX_CONJUNCTS_TO_INJECT)
 					{
 						AutoPtr<ComparativeBoolNode> cmpNode(FB_NEW_POOL(getPool())
 							ComparativeBoolNode(getPool(), blr_eql));
@@ -2081,7 +2214,7 @@ unsigned Optimizer::distributeEqualities(BoolExprNodeStack& orgStack, unsigned b
 			{
 				for (ValueExprNodeStack::iterator temp(*eq_class); temp.hasData(); ++temp)
 				{
-					if (!fieldEqual(node1, temp.object()) && count < baseCount)
+					if (!fieldEqual(node1, temp.object()) && count < MAX_CONJUNCTS_TO_INJECT)
 					{
 						ValueExprNode* arg1;
 						ValueExprNode* arg2;
@@ -2234,16 +2367,38 @@ void Optimizer::formRivers(const StreamList& streams,
 // If the whole things is a moby no-op, return false.
 //
 
-bool Optimizer::generateEquiJoin(RiverList& orgRivers)
+bool Optimizer::generateEquiJoin(RiverList& rivers, JoinType joinType)
 {
+	fb_assert(joinType != OUTER_JOIN);
+
 	ULONG selected_rivers[OPT_STREAM_BITS], selected_rivers2[OPT_STREAM_BITS];
 	ValueExprNode** eq_class;
 
+	RiverList orgRivers(rivers);
+
+	// Find dependent rivers and exclude them from processing
+
+	for (River** iter = orgRivers.begin(); iter < orgRivers.end();)
+	{
+		const auto river = *iter;
+
+		StreamStateHolder stateHolder2(csb, river->getStreams());
+		stateHolder2.activate();
+
+		if (river->isComputable(csb))
+		{
+			iter++;
+			continue;
+		}
+
+		orgRivers.remove(iter);
+	}
+
 	// Count the number of "rivers" involved in the operation, then allocate
 	// a scratch block large enough to hold values to compute equality
 	// classes.
 
-	const unsigned orgCount = (unsigned) orgRivers.getCount();
+	const auto orgCount = (unsigned) orgRivers.getCount();
 
 	if (orgCount < 2)
 		return false;
@@ -2263,6 +2418,9 @@ bool Optimizer::generateEquiJoin(RiverList& orgRivers)
 		if (iter & CONJUNCT_USED)
 			continue;
 
+		if (!iter->computable(csb, INVALID_STREAM, false))
+			continue;
+
 		NestConst<ValueExprNode> node1;
 		NestConst<ValueExprNode> node2;
 
@@ -2347,7 +2505,7 @@ bool Optimizer::generateEquiJoin(RiverList& orgRivers)
 	// Prepare rivers for joining
 
 	StreamList streams;
-	RiverList rivers;
+	RiverList joinedRivers;
 	HalfStaticArray<NestValueArray*, OPT_STATIC_ITEMS> keys;
 	unsigned position = 0, maxCardinalityPosition = 0, lowestPosition = MAX_ULONG;
 	double maxCardinality1 = 0, maxCardinality2 = 0;
@@ -2376,13 +2534,13 @@ bool Optimizer::generateEquiJoin(RiverList& orgRivers)
 		{
 			maxCardinality2 = maxCardinality1;
 			maxCardinality1 = cardinality;
-			maxCardinalityPosition = rivers.getCount();
+			maxCardinalityPosition = joinedRivers.getCount();
 		}
 		else if (cardinality > maxCardinality2)
 			maxCardinality2 = cardinality;
 
 		streams.join(river->getStreams());
-		rivers.add(river);
+		joinedRivers.add(river);
 		orgRivers.remove(iter);
 
 		// Collect keys to join on
@@ -2405,10 +2563,11 @@ bool Optimizer::generateEquiJoin(RiverList& orgRivers)
 	HalfStaticArray<RecordSource*, OPT_STATIC_ITEMS> rsbs;
 	RecordSource* finalRsb = nullptr;
 
-	if (useMergeJoin)
+	// MERGE JOIN does not support other join types yet
+	if (useMergeJoin && joinType == INNER_JOIN)
 	{
 		position = 0;
-		for (const auto river : rivers)
+		for (const auto river : joinedRivers)
 		{
 			const auto sort = FB_NEW_POOL(getPool()) SortNode(getPool());
 
@@ -2432,29 +2591,36 @@ bool Optimizer::generateEquiJoin(RiverList& orgRivers)
 	}
 	else
 	{
-		// Ensure that the largest river is placed at the first position.
-		// It's important for a hash join to be efficient.
+		if (joinType == INNER_JOIN)
+		{
+			// Ensure that the largest river is placed at the first position.
+			// It's important for a hash join to be efficient.
 
-		const auto maxCardinalityRiver = rivers[maxCardinalityPosition];
-		rivers[maxCardinalityPosition] = rivers[0];
-		rivers[0] = maxCardinalityRiver;
+			const auto maxCardinalityRiver = joinedRivers[maxCardinalityPosition];
+			joinedRivers[maxCardinalityPosition] = joinedRivers[0];
+			joinedRivers[0] = maxCardinalityRiver;
 
-		const auto maxCardinalityKey = keys[maxCardinalityPosition];
-		keys[maxCardinalityPosition] = keys[0];
-		keys[0] = maxCardinalityKey;
+			const auto maxCardinalityKey = keys[maxCardinalityPosition];
+			keys[maxCardinalityPosition] = keys[0];
+			keys[0] = maxCardinalityKey;
+		}
 
-		for (const auto river : rivers)
+		for (const auto river : joinedRivers)
 			rsbs.add(river->getRecordSource());
 
 		finalRsb = FB_NEW_POOL(getPool())
-			HashJoin(tdbb, csb, rsbs.getCount(), rsbs.begin(), keys.begin());
+			HashJoin(tdbb, csb, joinType, rsbs.getCount(), rsbs.begin(), keys.begin());
 	}
 
 	// Pick up any boolean that may apply
 	finalRsb = applyLocalBoolean(finalRsb, streams, iter);
 
-	const auto finalRiver = FB_NEW_POOL(getPool()) River(csb, finalRsb, rivers);
-	orgRivers.insert(lowestPosition, finalRiver);
+	const auto finalRiver = FB_NEW_POOL(getPool()) River(csb, finalRsb, joinedRivers);
+
+	for (const auto river : joinedRivers)
+		rivers.findAndRemove(river);
+
+	rivers.insert(lowestPosition, finalRiver);
 
 	return true;
 }
@@ -2577,8 +2743,7 @@ RecordSource* Optimizer::generateOuterJoin(RiverList& rivers,
 
 		// Allocate and fill in the rsb
 		return FB_NEW_POOL(getPool())
-			NestedLoopJoin(csb, stream_o.stream_rsb, stream_i.stream_rsb,
-						   boolean, OUTER_JOIN);
+			NestedLoopJoin(csb, stream_o.stream_rsb, stream_i.stream_rsb, boolean);
 	}
 
 	bool hasOuterRsb = true, hasInnerRsb = true;
@@ -2601,7 +2766,7 @@ RecordSource* Optimizer::generateOuterJoin(RiverList& rivers,
 	const auto innerRsb = generateResidualBoolean(stream_i.stream_rsb);
 
 	const auto rsb1 = FB_NEW_POOL(getPool())
-		NestedLoopJoin(csb, stream_o.stream_rsb, innerRsb, boolean, OUTER_JOIN);
+		NestedLoopJoin(csb, stream_o.stream_rsb, innerRsb, boolean);
 
 	for (auto iter = getConjuncts(); iter.hasData(); ++iter)
 	{
@@ -2631,10 +2796,13 @@ RecordSource* Optimizer::generateOuterJoin(RiverList& rivers,
 
 	const auto outerRsb = generateResidualBoolean(stream_o.stream_rsb);
 
-	const auto rsb2 = FB_NEW_POOL(getPool())
-		NestedLoopJoin(csb, stream_i.stream_rsb, outerRsb, boolean, ANTI_JOIN);
+	StreamList outerStreams;
+	outerRsb->findUsedStreams(outerStreams);
 
-	return FB_NEW_POOL(getPool()) FullOuterJoin(csb, rsb1, rsb2);
+	const auto rsb2 = FB_NEW_POOL(getPool())
+		NestedLoopJoin(csb, stream_i.stream_rsb, outerRsb, boolean);
+
+	return FB_NEW_POOL(getPool()) FullOuterJoin(csb, rsb1, rsb2, outerStreams);
 }
 
 
@@ -2860,41 +3028,6 @@ RecordSource* Optimizer::generateRetrieval(StreamType stream,
 }
 
 
-//
-// Find conjuncts local to the given river and compose an appropriate filter
-//
-
-RecordSource* Optimizer::applyLocalBoolean(RecordSource* rsb,
-										   const StreamList& streams,
-										   ConjunctIterator& iter)
-{
-	StreamStateHolder globalHolder(csb);
-	globalHolder.deactivate();
-
-	StreamStateHolder localHolder(csb, streams);
-	localHolder.activate(csb);
-
-	BoolExprNode* boolean = nullptr;
-	double selectivity = MAXIMUM_SELECTIVITY;
-
-	for (iter.rewind(); iter.hasData(); ++iter)
-	{
-		if (!(iter & CONJUNCT_USED) &&
-			!(iter->nodFlags & ExprNode::FLAG_RESIDUAL) &&
-			iter->computable(csb, INVALID_STREAM, false))
-		{
-			compose(getPool(), &boolean, iter);
-			iter |= CONJUNCT_USED;
-
-			if (!(iter & (CONJUNCT_MATCHED | CONJUNCT_JOINED)))
-				selectivity *= getSelectivity(*iter);
-		}
-	}
-
-	return boolean ? FB_NEW_POOL(getPool()) FilteredStream(csb, rsb, boolean, selectivity) : rsb;
-}
-
-
 //
 // Check whether the given boolean can be involved in a equi-join relationship
 //
@@ -3250,6 +3383,8 @@ ValueExprNode* Optimizer::optimizeLikeSimilar(ComparativeBoolNode* cmpNode)
 
 		MoveBuffer prefixBuffer;
 		ULONG charLen = 0;
+		bool specialCharFound = false;
+		FB_SIZE_T prevPrefixSize = 0;
 
 		while (IntlUtil::readOneChar(matchCharset, &patternPtr, patternEnd, &charLen))
 		{
@@ -3264,9 +3399,36 @@ ValueExprNode* Optimizer::optimizeLikeSimilar(ComparativeBoolNode* cmpNode)
 				}
 			}
 			else if (charLen == 1 && SimilarToRegex::isSpecialChar(*patternPtr))
-				break;
+			{
+				const auto patternChar = *patternPtr;
 
-			prefixBuffer.push(patternPtr, charLen);
+				// If there are any branches, we assume there is no commom prefix.
+				if (patternChar == '|')
+					return nullptr;
+
+				if (!specialCharFound)
+				{
+					switch (patternChar)
+					{
+						// These patterns may make the previous char optional.
+						case '*':
+						case '?':
+						case '{':
+							prefixBuffer.resize(prevPrefixSize);
+							break;
+					}
+
+					specialCharFound = true;
+				}
+
+				break;
+			}
+
+			if (!specialCharFound)
+			{
+				prevPrefixSize = prefixBuffer.getCount();
+				prefixBuffer.push(patternPtr, charLen);
+			}
 		}
 
 		if (prefixBuffer.isEmpty())
