@@ -1100,7 +1100,8 @@ void PAG_header(thread_db* tdbb, bool info)
 	fb_assert(attachment);
 
 	WIN window(HEADER_PAGE_NUMBER);
-	header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
+	pag* page = CCH_FETCH(tdbb, &window, LCK_read, pag_header);
+	header_page* header = (header_page*) page;
 
 	try {
 
@@ -1220,6 +1221,30 @@ void PAG_header(thread_db* tdbb, bool info)
 			fb_assert(false);
 	}
 
+	// If database in backup lock state...
+	if (!info && dbb->dbb_backup_manager->getState() != Ods::hdr_nbak_normal)
+	{
+		// refetch some data from the header, because it could be changed in the delta file
+		// (as initially PAG_init2 reads the header from the main file and these values
+		// may be outdated there)
+		for (const UCHAR* p = header->hdr_data; *p != HDR_end; p += 2u + p[1])
+		{
+			switch (*p)
+			{
+			case HDR_sweep_interval:
+				fb_assert(p[1] == sizeof(SLONG));
+				memcpy(&dbb->dbb_sweep_interval, p + 2, sizeof(SLONG));
+				break;
+
+			case HDR_repl_seq:
+				fb_assert(p[1] == sizeof(FB_UINT64));
+				memcpy(&dbb->dbb_repl_sequence, p + 2, sizeof(FB_UINT64));
+				break;
+
+			}
+		}
+	}
+
 	}	// try
 	catch (const Exception&)
 	{
@@ -1261,10 +1286,13 @@ void PAG_header_init(thread_db* tdbb)
 	// and unit of transfer is a multiple of physical disk
 	// sector for raw disk access.
 
-	UCHAR temp_buffer[RAW_HEADER_SIZE + PAGE_ALIGNMENT];
-	UCHAR* const temp_page = FB_ALIGN(temp_buffer, PAGE_ALIGNMENT);
+	const ULONG ioBlockSize = dbb->getIOBlockSize();
+	const ULONG headerSize = MAX(RAW_HEADER_SIZE, ioBlockSize);
 
-	PIO_header(tdbb, temp_page, RAW_HEADER_SIZE);
+	HalfStaticArray<UCHAR, RAW_HEADER_SIZE + PAGE_ALIGNMENT> temp;
+	UCHAR* const temp_page = temp.getAlignedBuffer(headerSize, ioBlockSize);
+
+	PIO_header(tdbb, temp_page, headerSize);
 	const header_page* header = (header_page*) temp_page;
 
 	if (header->hdr_header.pag_type != pag_header || header->hdr_sequence)
@@ -1377,8 +1405,7 @@ void PAG_init2(thread_db* tdbb, USHORT shadow_number)
 	// the temporary page buffer for raw disk access.
 
 	Array<UCHAR> temp;
-	UCHAR* const temp_page =
-		FB_ALIGN(temp.getBuffer(dbb->dbb_page_size + PAGE_ALIGNMENT), PAGE_ALIGNMENT);
+	UCHAR* const temp_page = temp.getAlignedBuffer(dbb->dbb_page_size, dbb->getIOBlockSize());
 
 	PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
 	jrd_file* file = pageSpace->file;
@@ -2603,7 +2630,7 @@ ULONG PAG_page_count(thread_db* tdbb)
 	Database* const dbb = tdbb->getDatabase();
 	Array<UCHAR> temp;
 	page_inv_page* pip = reinterpret_cast<Ods::page_inv_page*>
-		(FB_ALIGN(temp.getBuffer(dbb->dbb_page_size + PAGE_ALIGNMENT), PAGE_ALIGNMENT));
+		(temp.getAlignedBuffer(dbb->dbb_page_size, dbb->getIOBlockSize()));
 
 	PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
 	fb_assert(pageSpace);
