@@ -201,11 +201,12 @@ namespace
 		}
 	};
 
-	class LocalThreadContext
+	class LocalThreadContext : Firebird::ContextPoolHolder
 	{
 	public:
 		LocalThreadContext(thread_db* tdbb, jrd_tra* tra, Request* req = NULL)
-			: m_tdbb(tdbb)
+			: Firebird::ContextPoolHolder(req ? req->req_pool : tdbb->getDefaultPool()),
+			  m_tdbb(tdbb)
 		{
 			tdbb->setTransaction(tra);
 			tdbb->setRequest(req);
@@ -271,16 +272,20 @@ Applier* Applier::create(thread_db* tdbb)
 
 void Applier::shutdown(thread_db* tdbb)
 {
+	const auto dbb = tdbb->getDatabase();
 	const auto attachment = tdbb->getAttachment();
 
-	cleanupTransactions(tdbb);
-
-	CMP_release(tdbb, m_request);
+	if (!(dbb->dbb_flags & DBB_bugcheck))
+	{
+		cleanupTransactions(tdbb);
+		CMP_release(tdbb, m_request);
+	}
 	m_request = nullptr;	// already deleted by pool
 	m_record = nullptr;		// already deleted by pool
 	m_bitmap = nullptr;		// already deleted by pool
 
-	attachment->att_repl_appliers.findAndRemove(this);
+	if (attachment)
+		attachment->att_repl_appliers.findAndRemove(this);
 
 	if (m_interface)
 	{
@@ -568,6 +573,8 @@ void Applier::insertRecord(thread_db* tdbb, TraNumber traNum,
 	rpb.rpb_length = length;
 	record->copyDataFrom(data);
 
+	FbLocalStatus error;
+
 	try
 	{
 		doInsert(tdbb, &rpb, transaction);
@@ -582,6 +589,7 @@ void Applier::insertRecord(thread_db* tdbb, TraNumber traNum,
 			throw;
 		}
 
+		ex.stuffException(&error);
 		fb_utils::init_status(tdbb->tdbb_status_vector);
 
 		// The subsequent backout will delete the blobs we have stored before,
@@ -622,8 +630,14 @@ void Applier::insertRecord(thread_db* tdbb, TraNumber traNum,
 	bool found = false;
 
 #ifdef RESOLVE_CONFLICTS
+	fb_assert(error[1] == isc_unique_key_violation || error[1] == isc_no_dup);
+	fb_assert(error[2] == isc_arg_string);
+	fb_assert(error[3] != 0);
+
+	const char* idxName = reinterpret_cast<const char*>(error[3]);
+
 	index_desc idx;
-	const auto indexed = lookupRecord(tdbb, relation, record, idx);
+	const auto indexed = lookupRecord(tdbb, relation, record, idx, idxName);
 
 	AutoPtr<Record> cleanup;
 
@@ -665,6 +679,12 @@ void Applier::insertRecord(thread_db* tdbb, TraNumber traNum,
 	}
 	else
 	{
+		fb_assert(rpb.rpb_record == record);
+
+		rpb.rpb_format_number = format->fmt_version;
+		rpb.rpb_address = record->getData();
+		rpb.rpb_length = record->getLength();
+
 		doInsert(tdbb, &rpb, transaction); // second (paranoid) attempt
 	}
 }
@@ -971,6 +991,7 @@ void Applier::executeSql(thread_db* tdbb,
 
 	UserId* const owner = attachment->getUserId(ownerName);
 	AutoSetRestore<UserId*> autoOwner(&attachment->att_ss_user, owner);
+	AutoSetRestore<UserId*> autoUser(&attachment->att_user, owner);
 	AutoSetRestoreFlag<ULONG> noCascade(&tdbb->tdbb_flags, TDBB_repl_in_progress, !m_enableCascade);
 
 	DSQL_execute_immediate(tdbb, attachment, &transaction,
@@ -1066,7 +1087,7 @@ bool Applier::compareKey(thread_db* tdbb, jrd_rel* relation, const index_desc& i
 
 bool Applier::lookupRecord(thread_db* tdbb,
 						   jrd_rel* relation, Record* record,
-						   index_desc& idx)
+						   index_desc& idx, const char* idxName)
 {
 	RecordBitmap::reset(m_bitmap);
 
@@ -1077,7 +1098,24 @@ bool Applier::lookupRecord(thread_db* tdbb,
 		return false;
 	}
 
-	if (lookupKey(tdbb, relation, idx))
+	bool haveIdx = false;
+	if (idxName)
+	{
+		SLONG foundRelId;
+		IndexStatus idxStatus;
+		SLONG idx_id = MET_lookup_index_name(tdbb, idxName, &foundRelId, &idxStatus);
+
+		fb_assert(idxStatus == MET_object_active);
+		fb_assert(foundRelId == relation->rel_id);
+
+		haveIdx = (idxStatus == MET_object_active) && (foundRelId == relation->rel_id) &&
+			BTR_lookup(tdbb, relation, idx_id, &idx, relation->getPages(tdbb));
+	}
+
+	if (!haveIdx)
+		haveIdx = lookupKey(tdbb, relation, idx);
+
+	if (haveIdx)
 	{
 		IndexKey key(tdbb, relation, &idx);
 		if (const auto result = key.compose(record))
