@@ -90,8 +90,7 @@ ConfigStorage::ConfigStorage()
 	  m_filename(getPool()),
 	  m_recursive(0),
 	  m_mutexTID(0),
-	  m_dirty(false),
-	  m_nextIdx(0)
+	  m_dirty(false)
 {
 #ifdef WIN_NT
 	DWORD sesID = 0;
@@ -164,6 +163,7 @@ void ConfigStorage::shutdown()
 
 	{
 		StorageGuard guard(this);
+		fb_assert(m_sharedMemory->getHeader()->cnt_uses != 0);
 		--(m_sharedMemory->getHeader()->cnt_uses);
 		if (m_sharedMemory->getHeader()->cnt_uses == 0)
 		{
@@ -470,13 +470,20 @@ void ConfigStorage::compact()
 	ULONG check_used, check_size;
 	check_used = check_size = sizeof(TraceCSHeader);
 
+	// Track undeleted slots from dead processes
+	Firebird::SortedArray<ULONG, InlineStorage<ULONG, 16>> deadProcesses;
+
 	// collect used slots, sort them by offset
 	for (TraceCSHeader::Slot* slot = header->slots; slot < header->slots + header->slots_cnt; slot++)
 	{
 		if (slot->used && slot->ses_pid != pid &&
+			((slot->ses_flags & trs_system) == 0) && // System sessions are shared for multiple connections so they may live without the original process
 			!ISC_check_process_existence(slot->ses_pid))
 		{
-			header->cnt_uses--; // Process that created trace session disappeared, count it out
+			// A SUPER server may shut down, but its Storage shared memory continues to live due to an embedded user session.
+			// The process might allocate multiple slots, so count them carefully.
+			deadProcesses.add(slot->ses_pid);
+
 			markDeleted(slot);
 		}
 
@@ -489,6 +496,11 @@ void ConfigStorage::compact()
 		data.add(item);
 	}
 
+	// Process that created storages disappeared, count it out
+	fb_assert(header->cnt_uses > deadProcesses.getCount());
+	header->cnt_uses -= deadProcesses.getCount();
+	deadProcesses.clear();
+
 	fb_assert(check_used == header->mem_used);
 	fb_assert(check_size == header->mem_offset);
 
@@ -744,19 +756,14 @@ bool ConfigStorage::getSession(Firebird::TraceSession& session, GET_FLAGS getFla
 	return readSession(slot, session, getFlag);
 }
 
-void ConfigStorage::restart()
-{
-	m_nextIdx = 0;
-}
-
-bool ConfigStorage::getNextSession(TraceSession& session, GET_FLAGS getFlag)
+bool ConfigStorage::getNextSession(TraceSession& session, GET_FLAGS getFlag, ULONG& nextIdx)
 {
 	TraceCSHeader* header = m_sharedMemory->getHeader();
 
-	while (m_nextIdx < header->slots_cnt)
+	while (nextIdx < header->slots_cnt)
 	{
-		TraceCSHeader::Slot* slot = header->slots + m_nextIdx;
-		m_nextIdx++;
+		TraceCSHeader::Slot* slot = header->slots + nextIdx;
+		nextIdx++;
 
 		if (slot->used)
 			return readSession(slot, session, getFlag);
@@ -895,6 +902,31 @@ void ConfigStorage::updateFlags(TraceSession& session)
 	slot->ses_flags = session.ses_flags;
 }
 
+bool ConfigStorage::Accessor::getNext(TraceSession& session, GET_FLAGS getFlag)
+{
+	if (m_guard)
+		return m_storage->getNextSession(session, getFlag, m_nextIdx);
+
+	StorageGuard guard(m_storage);
+
+	// Restore position, if required: find index of slot with session ID greater than m_sesId.
+	if (m_change_number != m_storage->getChangeNumber())
+	{
+		if (m_storage->findSession(m_sesId, m_nextIdx))
+			m_nextIdx++;
+
+		m_change_number = m_storage->getChangeNumber();
+	}
+
+	if (m_storage->getNextSession(session, getFlag, m_nextIdx))
+	{
+		m_sesId = session.ses_id;
+		return true;
+	}
+
+	return false;
+}
+
 void ConfigStorage::Writer::write(ITEM tag, ULONG len, const void* data)
 {
 	if (m_mem + 1 > m_end)
