@@ -1904,7 +1904,7 @@ Validation::RTN Validation::walk_data_page(jrd_rel* relation, ULONG page_number,
 						if (!getInfo.m_condition)
 							getInfo.m_condition = FB_NEW_POOL(*pool) IndexCondition(vdr_tdbb, &getInfo.m_desc);
 
-						if (getInfo.m_condition->evaluate(rpb.rpb_record))
+						if (getInfo.m_condition->check(rpb.rpb_record).value)
 							RBM_SET(pool, &getInfo.m_recs, recno);
 					}
 				}
@@ -1938,21 +1938,19 @@ void Validation::walk_generators()
 
 	WIN window(DB_PAGE_SPACE, -1);
 
-	vcl* vector = dbb->dbb_gen_id_pages;
-	if (vector)
+	if (const auto idsCount = dbb->getKnownPagesCount(pag_ids))
 	{
-        vcl::iterator ptr, end;
-		for (ptr = vector->begin(), end = vector->end(); ptr < end; ++ptr)
+		for (ULONG sequence = 0; sequence < idsCount; sequence++)
 		{
-			if (*ptr)
+			if (const auto pageNumber = dbb->getKnownPage(pag_ids, sequence))
 			{
 #ifdef DEBUG_VAL_VERBOSE
 				if (VAL_debug_level)
-					fprintf(stdout, "walk_generator: page %d\n", *ptr);
+					fprintf(stdout, "walk_generator: page %d\n", pageNumber);
 #endif
 				// It doesn't make a difference generator_page or pointer_page because it's not used.
 				generator_page* page = NULL;
-				fetch_page(true, *ptr, pag_ids, &window, &page);
+				fetch_page(true, pageNumber, pag_ids, &window, &page);
 				release_page(&window);
 			}
 		}
@@ -2789,21 +2787,45 @@ Validation::RTN Validation::walk_record(jrd_rel* relation, const rhd* header, US
 	if (header->rhd_flags & rhd_incomplete)
 	{
 		p = fragment->rhdf_data;
-		length -= offsetof(rhdf, rhdf_data[0]);
+		length -= RHDF_SIZE;
 	}
 	else if (header->rhd_flags & rhd_long_tranum)
 	{
 		p = ((rhde*) header)->rhde_data;
-		length -= offsetof(rhde, rhde_data[0]);
+		length -= RHDE_SIZE;
 	}
 	else
 	{
 		p = header->rhd_data;
-		length -= offsetof(rhd, rhd_data[0]);
+		length -= RHD_SIZE;
 	}
 
-	ULONG record_length = (header->rhd_flags & rhd_not_packed) ?
-		length : Compressor::getUnpackedLength(length, p);
+	const auto format = MET_format(vdr_tdbb, relation, header->rhd_format);
+	auto remainingLength = format->fmt_length;
+
+	auto calculateLength = [remainingLength](ULONG length, const UCHAR* data, bool notPacked)
+	{
+		if (notPacked)
+		{
+			if (length > remainingLength)
+			{
+				// Short records may be zero-padded up to the fragmented header size.
+				// Find out how many zero bytes present inside the tail and adjust
+				// the calculated record length accordingly.
+
+				auto tail = data + remainingLength;
+				for (const auto end = data + length; tail < end && !*tail; tail++)
+					length--;
+			}
+
+			return length;
+		}
+
+		return Compressor::getUnpackedLength(length, data);
+	};
+
+	bool notPacked = (fragment->rhdf_flags & rhd_not_packed) != 0;
+	remainingLength -= calculateLength(length, p, notPacked);
 
 	// Next, chase down fragments, if any
 
@@ -2842,21 +2864,21 @@ Validation::RTN Validation::walk_record(jrd_rel* relation, const rhd* header, US
 		if (fragment->rhdf_flags & rhd_incomplete)
 		{
 			p = fragment->rhdf_data;
-			length -= offsetof(rhdf, rhdf_data[0]);
+			length -= RHDF_SIZE;
 		}
 		else if (fragment->rhdf_flags & rhd_long_tranum)
 		{
 			p = ((rhde*) fragment)->rhde_data;
-			length -= offsetof(rhde, rhde_data[0]);
+			length -= RHDE_SIZE;
 		}
 		else
 		{
 			p = ((rhd*) fragment)->rhd_data;
-			length -= offsetof(rhd, rhd_data[0]);
+			length -= RHD_SIZE;
 		}
 
-		record_length += (fragment->rhdf_flags & rhd_not_packed) ?
-			length : Compressor::getUnpackedLength(length, p);
+		notPacked = (fragment->rhdf_flags & rhd_not_packed) != 0;
+		remainingLength -= calculateLength(length, p, notPacked);
 
 		page_number = fragment->rhdf_f_page;
 		line_number = fragment->rhdf_f_line;
@@ -2864,11 +2886,9 @@ Validation::RTN Validation::walk_record(jrd_rel* relation, const rhd* header, US
 		release_page(&window);
 	}
 
-	// Check out record length and format
+	// Validate unpacked record length
 
-	const Format* format = MET_format(vdr_tdbb, relation, header->rhd_format);
-
-	if (!delta_flag && record_length != format->fmt_length)
+	if (!delta_flag && remainingLength != 0)
 		return corrupt(VAL_REC_WRONG_LENGTH, relation, number.getValue());
 
 	return rtn_ok;
@@ -3282,8 +3302,7 @@ Validation::RTN Validation::walk_tip(TraNumber transaction)
  **************************************/
 	Database* dbb = vdr_tdbb->getDatabase();
 
-	const vcl* vector = dbb->dbb_t_pages;
-	if (!vector)
+	if (!dbb->getKnownPagesCount(pag_transactions))
 		return corrupt(VAL_TIP_LOST, 0);
 
 	tx_inv_page* page = 0;
@@ -3291,25 +3310,28 @@ Validation::RTN Validation::walk_tip(TraNumber transaction)
 
 	for (ULONG sequence = 0; sequence <= pages; sequence++)
 	{
-		if (!(*vector)[sequence] || sequence >= vector->count())
+		auto pageNumber = dbb->getKnownPage(pag_transactions, sequence);
+		if (!pageNumber)
 		{
 			corrupt(VAL_TIP_LOST_SEQUENCE, 0, sequence);
 			if (!(vdr_flags & VDR_repair))
 				continue;
 
 			TRA_extend_tip(vdr_tdbb, sequence);
-			vector = dbb->dbb_t_pages;
 			vdr_fixed++;
+
+			pageNumber = dbb->getKnownPage(pag_transactions, sequence);
 		}
 
 		WIN window(DB_PAGE_SPACE, -1);
-		fetch_page(true, (*vector)[sequence], pag_transactions, &window, &page);
+		fetch_page(true, pageNumber, pag_transactions, &window, &page);
 
 #ifdef DEBUG_VAL_VERBOSE
 		if (VAL_debug_level)
-			fprintf(stdout, "walk_tip: page %d next %d\n", (*vector)[sequence], page->tip_next);
+			fprintf(stdout, "walk_tip: page %d next %d\n", pageNumber, page->tip_next);
 #endif
-		if (page->tip_next && page->tip_next != (*vector)[sequence + 1])
+		const auto next = dbb->getKnownPage(pag_transactions, sequence + 1);
+		if (page->tip_next && page->tip_next != next)
 		{
 			corrupt(VAL_TIP_CONFUSED, 0, sequence);
 		}
