@@ -237,7 +237,7 @@ rem_port* XNET_analyze(ClntAuthBlock* cBlock,
 					   const PathName& file_name,
 					   bool uv_flag,
 					   RefPtr<const Config>* config,
-					   const Firebird::PathName* ref_db_name)
+					   const PathName* ref_db_name)
 {
 /**************************************
  *
@@ -361,10 +361,10 @@ rem_port* XNET_analyze(ClntAuthBlock* cBlock,
 	case op_response:
 		try
 		{
-			Firebird::LocalStatus warning;		// Ignore connect warnings for a while
+			LocalStatus warning;		// Ignore connect warnings for a while
 			REMOTE_check_response(&warning, rdb, packet);
 		}
-		catch (const Firebird::Exception&)
+		catch (const Exception&)
 		{
 			disconnect(port);
 			delete rdb;
@@ -405,7 +405,7 @@ rem_port* XNET_analyze(ClntAuthBlock* cBlock,
 
 rem_port* XNET_connect(PACKET* packet,
 					   USHORT flag,
-					   Firebird::RefPtr<const Config>* config)
+					   RefPtr<const Config>* config)
 {
 /**************************************
  *
@@ -1108,7 +1108,7 @@ rem_port* XnetClientEndPoint::connect_client(PACKET* packet, const RefPtr<const
  *
  **************************************/
 
-	const Firebird::RefPtr<const Config>& conf(config ? *config : Config::getDefaultConfig());
+	const RefPtr<const Config>& conf(config ? *config : Config::getDefaultConfig());
 
 	if (!xnet_initialized)
 	{
@@ -1991,8 +1991,8 @@ static bool_t xnet_read(RemoteXdr* xdrs)
 		{
 			// Client has written some data for us (server) to read
 
-			port->port_rcv_packets++;
-			port->port_rcv_bytes += xch->xch_length;
+			port->bumpPhysStats(rem_port::RECEIVE, xch->xch_length);
+			port->bumpLogBytes(rem_port::RECEIVE, xch->xch_length);	// XNET not calls REMOTE_inflate
 
 			xdrs->x_handy = xch->xch_length;
 			xdrs->x_private = xdrs->x_base;
@@ -2048,8 +2048,8 @@ static bool_t xnet_write(RemoteXdr* xdrs)
 	xch->xch_length = xdrs->x_private - xdrs->x_base;
 	if (SetEvent(xcc->xcc_event_send_channel_filled))
 	{
-		port->port_snd_packets++;
-		port->port_snd_bytes += xch->xch_length;
+		port->bumpPhysStats(rem_port::SEND, xch->xch_length);
+		port->bumpLogBytes(rem_port::SEND, xch->xch_length);	// XNET not calls REMOTE_deflate
 
 		xdrs->x_private = xdrs->x_base;
 		xdrs->x_handy = xch->xch_size;
