@@ -100,7 +100,6 @@ enum SQL_STMT_TYPE
 };
 
 static bool alloc_cstring(RemoteXdr*, CSTRING*);
-static void free_cstring(RemoteXdr*, CSTRING*);
 static void reset_statement(RemoteXdr*, SSHORT);
 static bool_t xdr_cstring(RemoteXdr*, CSTRING*);
 static bool_t xdr_response(RemoteXdr*, CSTRING*);
@@ -304,6 +303,9 @@ bool_t xdr_protocol(RemoteXdr* xdrs, PACKET* p)
 
 	const auto port = xdrs->x_public;
 
+	if (xdrs->x_op != XDR_FREE)
+		port->bumpLogPackets(xdrs->x_op == XDR_ENCODE ? rem_port::SEND : rem_port::RECEIVE);
+
 	switch (p->p_operation)
 	{
 	case op_reject:
@@ -1028,7 +1030,7 @@ bool_t xdr_protocol(RemoteXdr* xdrs, PACKET* p)
 
 				if (xdrs->x_op == XDR_DECODE)
 				{
-					Firebird::Arg::StatusVector sv(ptr->value());
+					Arg::StatusVector sv(ptr->value());
 					LocalStatus to;
 					sv.copyTo(&to);
 					delete ptr;
@@ -1247,7 +1249,7 @@ static bool alloc_cstring(RemoteXdr* xdrs, CSTRING* cstring)
 
 	if (cstring->cstr_length > cstring->cstr_allocated && cstring->cstr_allocated)
 	{
-		free_cstring(xdrs, cstring);
+		cstring->free(xdrs);
 	}
 
 	if (!cstring->cstr_address)
@@ -1268,7 +1270,7 @@ static bool alloc_cstring(RemoteXdr* xdrs, CSTRING* cstring)
 }
 
 
-static void free_cstring( RemoteXdr* xdrs, CSTRING* cstring)
+void CSTRING::free(RemoteXdr* xdrs)
 {
 /**************************************
  *
@@ -1281,14 +1283,15 @@ static void free_cstring( RemoteXdr* xdrs, CSTRING* cstring)
  *
  **************************************/
 
-	if (cstring->cstr_allocated)
+	if (cstr_allocated)
 	{
-		delete[] cstring->cstr_address;
-		DEBUG_XDR_FREE(xdrs, cstring, cstring->cstr_address, cstring->cstr_allocated);
+		delete[] cstr_address;
+		if (xdrs)
+			DEBUG_XDR_FREE(xdrs, this, cstr_address, cstr_allocated);
 	}
 
-	cstring->cstr_address = NULL;
-	cstring->cstr_allocated = 0;
+	cstr_address = NULL;
+	cstr_allocated = 0;
 }
 
 
@@ -1393,7 +1396,7 @@ static bool_t xdr_cstring_with_limit( RemoteXdr* xdrs, CSTRING* cstring, ULONG l
 		return TRUE;
 
 	case XDR_FREE:
-		free_cstring(xdrs, cstring);
+		cstring->free(xdrs);
 		return TRUE;
 	}
 
@@ -1502,7 +1505,7 @@ static bool_t xdr_longs( RemoteXdr* xdrs, CSTRING* cstring)
 		break;
 
 	case XDR_FREE:
-		free_cstring(xdrs, cstring);
+		cstring->free(xdrs);
 		return TRUE;
 	}
 
@@ -1808,8 +1811,8 @@ static bool_t xdr_slice(RemoteXdr* xdrs, lstring* slice, /*USHORT sdl_length,*/
 
 	struct sdl_info info;
 	{
-		Firebird::LocalStatus ls;
-		Firebird::CheckStatusWrapper s(&ls);
+		LocalStatus ls;
+		CheckStatusWrapper s(&ls);
 		if (SDL_info(&s, sdl, &info, 0))
 			return FALSE;
 	}
