@@ -136,7 +136,8 @@ const USHORT ptype_lazy_send	= 5;	// Deferred packets delivery
 const USHORT ptype_MASK			= 0xFF;	// Mask - up to 255 types of protocol
 //
 // upper byte is used for protocol flags
-const USHORT pflag_compress		= 0x100;	// Turn on compression if possible
+const USHORT pflag_compress			= 0x100;	// Turn on compression if possible
+const USHORT pflag_win_sspi_nego	= 0x200;	// Win_SSPI supports Negotiate security package
 
 // Generic object id
 
@@ -324,11 +325,15 @@ enum P_OP
 
 // Count String Structure
 
+struct RemoteXdr;
+
 typedef struct cstring
 {
 	ULONG	cstr_length;
 	ULONG	cstr_allocated;
 	UCHAR*	cstr_address;
+
+	void	free(RemoteXdr* xdrs = nullptr);
 } CSTRING;
 
 // CVC: Only used in p_blob, p_sgmt & p_ddl, to validate constness.
