@@ -38,9 +38,12 @@
 #include "../common/os/mod_loader.h"
 #include "../jrd/license.h"
 #include "../common/classes/ImplementHelper.h"
+#include "../common/utils_proto.h"
+
+using namespace Firebird;
 
 #ifdef DEV_BUILD
-Firebird::AtomicCounter rem_port::portCounter;
+AtomicCounter rem_port::portCounter;
 #endif
 
 #ifdef REMOTE_DEBUG
@@ -350,7 +353,7 @@ void REMOTE_free_packet( rem_port* port, PACKET * packet, bool partial)
 }
 
 
-void REMOTE_get_timeout_params(rem_port* port, Firebird::ClumpletReader* pb)
+void REMOTE_get_timeout_params(rem_port* port, ClumpletReader* pb)
 {
 /**************************************
  *
@@ -600,14 +603,14 @@ void rem_port::linkParent(rem_port* const parent)
 	parent->port_clients = parent->port_next = this;
 }
 
-const Firebird::RefPtr<const Firebird::Config>& rem_port::getPortConfig() const
+const RefPtr<const Config>& rem_port::getPortConfig() const
 {
-	return port_config.hasData() ? port_config : Firebird::Config::getDefaultConfig();
+	return port_config.hasData() ? port_config : Config::getDefaultConfig();
 }
 
-Firebird::RefPtr<const Firebird::Config> rem_port::getPortConfig()
+RefPtr<const Config> rem_port::getPortConfig()
 {
-	return port_config.hasData() ? port_config : Firebird::Config::getDefaultConfig();
+	return port_config.hasData() ? port_config : Config::getDefaultConfig();
 }
 
 void rem_port::unlinkParent()
@@ -740,7 +743,7 @@ bool_t REMOTE_getbytes (RemoteXdr* xdrs, SCHAR* buff, unsigned bytecount)
 		}
 
 		rem_port* port = xdrs->x_public;
-		Firebird::RefMutexEnsureUnlock queGuard(*port->port_que_sync, FB_FUNCTION);
+		RefMutexEnsureUnlock queGuard(*port->port_que_sync, FB_FUNCTION);
 		queGuard.enter();
 		if (port->port_qoffset >= port->port_queue.getCount())
 		{
@@ -762,14 +765,14 @@ bool_t REMOTE_getbytes (RemoteXdr* xdrs, SCHAR* buff, unsigned bytecount)
 
 void PortsCleanup::registerPort(rem_port* port)
 {
-	Firebird::MutexLockGuard guard(m_mutex, FB_FUNCTION);
+	MutexLockGuard guard(m_mutex, FB_FUNCTION);
 
 	if (closing)
 		return;
 
 	if (!m_ports)
 	{
-		Firebird::MemoryPool& pool = *getDefaultMemoryPool();
+		MemoryPool& pool = *getDefaultMemoryPool();
 		m_ports = FB_NEW_POOL (pool) PortsArray(pool);
 	}
 
@@ -778,7 +781,7 @@ void PortsCleanup::registerPort(rem_port* port)
 
 void PortsCleanup::unRegisterPort(rem_port* port)
 {
-	Firebird::MutexLockGuard guard(m_mutex, FB_FUNCTION);
+	MutexLockGuard guard(m_mutex, FB_FUNCTION);
 
 	if (closing)
 		return;
@@ -798,11 +801,11 @@ void PortsCleanup::closePorts()
 	if (m_ports)
 		delay();
 
-	Firebird::MutexLockGuard guard(m_mutex, FB_FUNCTION);
-	Firebird::AutoSetRestore cl(&closing, true);
+	MutexLockGuard guard(m_mutex, FB_FUNCTION);
+	AutoSetRestore cl(&closing, true);
 
 	{ // scope
-		Firebird::MutexUnlockGuard g2(m_mutex, FB_FUNCTION);
+		MutexUnlockGuard g2(m_mutex, FB_FUNCTION);
 		Thread::yield();
 	}
 
@@ -855,18 +858,100 @@ ISC_STATUS* Rdb::get_status_vector() throw()
 }
 */
 
-void Rrq::saveStatus(const Firebird::Exception& ex) throw()
+
+bool RBlobInfo::getLocalInfo(unsigned int itemsLength, const unsigned char* items,
+	unsigned int bufferLength, unsigned char* buffer)
+{
+	if (!valid)
+		return false;
+
+	unsigned char* p = buffer;
+	const unsigned char* const end = buffer + bufferLength;
+
+	for (auto item = items; p && (item < items + itemsLength); item++)
+	{
+		if (*item == isc_info_end)
+			break;
+
+		switch (*item)
+		{
+		case isc_info_blob_num_segments:
+			p = fb_utils::putInfoItemInt(*item, num_segments, p, end);
+			break;
+
+		case isc_info_blob_max_segment:
+			p = fb_utils::putInfoItemInt(*item, max_segment, p, end);
+			break;
+
+		case isc_info_blob_total_length:
+			p = fb_utils::putInfoItemInt(*item, total_length, p, end);
+			break;
+
+		case isc_info_blob_type:
+			p = fb_utils::putInfoItemInt(*item, blob_type, p, end);
+			break;
+
+		default:
+			// unknown info item, let remote server handle it
+			return false;
+		}
+	}
+
+	if (p < end)
+		*p++ = isc_info_end;
+
+	return true;
+}
+
+
+void RBlobInfo::parseInfo(unsigned int bufferLength, const unsigned char* buffer)
+{
+	int c = 0;
+	valid = false;
+
+	ClumpletReader p(ClumpletReader::InfoResponse, buffer, bufferLength);
+	for (; !p.isEof(); p.moveNext())
+	{
+		switch (p.getClumpTag())
+		{
+		case isc_info_blob_num_segments:
+			num_segments = p.getInt();
+			c++;
+			break;
+		case isc_info_blob_max_segment:
+			max_segment = p.getInt();
+			c++;
+			break;
+		case isc_info_blob_total_length:
+			total_length = p.getInt();
+			c++;
+			break;
+		case isc_info_blob_type:
+			blob_type = p.getInt();
+			c++;
+			break;
+		case isc_info_end:
+			break;
+		default:
+			fb_assert(false);
+			break;
+		}
+	}
+	valid = (c == 4);
+}
+
+void Rrq::saveStatus(const Exception& ex) throw()
 {
 	if (rrqStatus.isSuccess())
 	{
-		Firebird::LocalStatus ls;
-		Firebird::CheckStatusWrapper tmp(&ls);
+		LocalStatus ls;
+		CheckStatusWrapper tmp(&ls);
 		ex.stuffException(&tmp);
 		rrqStatus.save(&tmp);
 	}
 }
 
-void Rrq::saveStatus(Firebird::IStatus* v) throw()
+void Rrq::saveStatus(IStatus* v) throw()
 {
 	if (rrqStatus.isSuccess())
 	{
@@ -874,28 +959,28 @@ void Rrq::saveStatus(Firebird::IStatus* v) throw()
 	}
 }
 
-void Rsr::saveException(const Firebird::Exception& ex, bool overwrite)
+void Rsr::saveException(const Exception& ex, bool overwrite)
 {
 	if (!rsr_status) {
-		rsr_status = FB_NEW Firebird::StatusHolder();
+		rsr_status = FB_NEW StatusHolder();
 	}
 
 	if (overwrite || !rsr_status->getError())
 	{
-		Firebird::LocalStatus ls;
-		Firebird::CheckStatusWrapper temp(&ls);
+		LocalStatus ls;
+		CheckStatusWrapper temp(&ls);
 		ex.stuffException(&temp);
 		rsr_status->save(&temp);
 	}
 }
 
-Firebird::string rem_port::getRemoteId() const
+string rem_port::getRemoteId() const
 {
 	fb_assert(port_protocol_id.hasData());
-	Firebird::string id = port_protocol_id;
+	string id = port_protocol_id;
 
 	if (port_address.hasData())
-		id += Firebird::string("/") + port_address;
+		id += string("/") + port_address;
 
 	return id;
 }
@@ -918,14 +1003,14 @@ LegacyPlugin REMOTE_legacy_auth(const char* nm, int p)
 	return PLUGIN_NEW;
 }
 
-Firebird::PathName ClntAuthBlock::getPluginName()
+PathName ClntAuthBlock::getPluginName()
 {
 	return plugins.hasData() ? plugins.name() : "";
 }
 
 template <typename T>
 static void addMultiPartConnectParameter(const T& dataToAdd,
-	Firebird::ClumpletWriter& user_id, UCHAR param)
+	ClumpletWriter& user_id, UCHAR param)
 {
 	FB_SIZE_T remaining = dataToAdd.getCount();
 	fb_assert(remaining <= 254u * 256u); // paranoid check => 65024
@@ -951,7 +1036,7 @@ static void addMultiPartConnectParameter(const T& dataToAdd,
 	}
 }
 
-void ClntAuthBlock::extractDataFromPluginTo(Firebird::ClumpletWriter& user_id)
+void ClntAuthBlock::extractDataFromPluginTo(ClumpletWriter& user_id)
 {
 	// Add user login name
 	if (cliOrigUserName.hasData())
@@ -962,7 +1047,7 @@ void ClntAuthBlock::extractDataFromPluginTo(Firebird::ClumpletWriter& user_id)
 	}
 
 	// Add plugin name
-	Firebird::PathName pluginName = getPluginName();
+	PathName pluginName = getPluginName();
 	if (pluginName.hasData())
 	{
 		HANDSHAKE_DEBUG(fprintf(stderr, "Cli: extractDataFromPluginTo: pluginName=%s\n", pluginName.c_str()));
@@ -982,7 +1067,7 @@ void ClntAuthBlock::extractDataFromPluginTo(Firebird::ClumpletWriter& user_id)
 	addMultiPartConnectParameter(dataFromPlugin, user_id, CNCT_specific_data);
 
 	// Client's wirecrypt requested level
-	user_id.insertInt(CNCT_client_crypt, clntConfig->getWireCrypt(Firebird::WC_CLIENT));
+	user_id.insertInt(CNCT_client_crypt, clntConfig->getWireCrypt(WC_CLIENT));
 }
 
 void ClntAuthBlock::resetClnt(const CSTRING* listStr)
@@ -995,7 +1080,7 @@ void ClntAuthBlock::resetClnt(const CSTRING* listStr)
 			return;
 		}
 
-		Firebird::ClumpletReader srvList(Firebird::ClumpletReader::UnTagged,
+		ClumpletReader srvList(ClumpletReader::UnTagged,
 										 listStr->cstr_address, listStr->cstr_length);
 
 		if (srvList.find(TAG_KNOWN_PLUGINS))
@@ -1009,18 +1094,18 @@ void ClntAuthBlock::resetClnt(const CSTRING* listStr)
 	firstTime = true;
 
 	pluginList = dpbPlugins.hasData() ? dpbPlugins :
-		clntConfig->getPlugins(Firebird::IPluginManager::TYPE_AUTH_CLIENT);
+		clntConfig->getPlugins(IPluginManager::TYPE_AUTH_CLIENT);
 
-	Firebird::PathName final;
+	PathName final;
 	if (serverPluginList.hasData())
 	{
-		Firebird::ParsedList::mergeLists(final, serverPluginList, pluginList);
+		ParsedList::mergeLists(final, serverPluginList, pluginList);
 		if (final.length() == 0)
 		{
 			HANDSHAKE_DEBUG(fprintf(stderr, "Cli: No matching plugins on client\n"));
-			(Firebird::Arg::Gds(isc_login)
+			(Arg::Gds(isc_login)
 #ifdef DEV_BUILD
-								<< Firebird::Arg::Gds(isc_random) << "No matching plugins on client"
+								<< Arg::Gds(isc_random) << "No matching plugins on client"
 #endif
 								).raise();
 		}
@@ -1033,7 +1118,7 @@ void ClntAuthBlock::resetClnt(const CSTRING* listStr)
 	plugins.set(final.c_str());
 }
 
-Firebird::RefPtr<const Firebird::Config>* ClntAuthBlock::getConfig()
+RefPtr<const Config>* ClntAuthBlock::getConfig()
 {
 	return clntConfig.hasData() ? &clntConfig : NULL;
 }
@@ -1044,25 +1129,25 @@ void ClntAuthBlock::storeDataForPlugin(unsigned int length, const unsigned char*
 	HANDSHAKE_DEBUG(fprintf(stderr, "Cli: accepted data for plugin length=%d\n", length));
 }
 
-Firebird::RefPtr<const Firebird::Config> REMOTE_get_config(const Firebird::PathName* dbName,
-	const Firebird::string* dpb_config)
+RefPtr<const Config> REMOTE_get_config(const PathName* dbName,
+	const string* dpb_config)
 {
-	Firebird::RefPtr<const Firebird::Config> config;
+	RefPtr<const Config> config;
 
 	if (dbName && dbName->hasData())
 	{
-		Firebird::PathName dummy;
+		PathName dummy;
 		expandDatabaseName(*dbName, dummy, &config);
 	}
 	else
-		config = Firebird::Config::getDefaultConfig();
+		config = Config::getDefaultConfig();
 
-	Firebird::Config::merge(config, dpb_config);
+	Config::merge(config, dpb_config);
 
 	return config;
 }
 
-void REMOTE_check_response(Firebird::IStatus* warning, Rdb* rdb, PACKET* packet, bool checkKeys)
+void REMOTE_check_response(IStatus* warning, Rdb* rdb, PACKET* packet, bool checkKeys)
 {
 /**************************************
  *
@@ -1078,7 +1163,7 @@ void REMOTE_check_response(Firebird::IStatus* warning, Rdb* rdb, PACKET* packet,
 	rdb->rdb_port->checkResponse(warning, packet, checkKeys);
 }
 
-void rem_port::checkResponse(Firebird::IStatus* warning, PACKET* packet, bool checkKeys)
+void rem_port::checkResponse(IStatus* warning, PACKET* packet, bool checkKeys)
 {
 /**************************************
  *
@@ -1102,7 +1187,7 @@ void rem_port::checkResponse(Firebird::IStatus* warning, PACKET* packet, bool ch
 
 	// Translate any gds codes into local operating specific codes
 
-	Firebird::StaticStatusVector newVector;
+	StaticStatusVector newVector;
 
 	while (*vector != isc_arg_end)
 	{
@@ -1142,7 +1227,7 @@ void rem_port::checkResponse(Firebird::IStatus* warning, PACKET* packet, bool ch
 	if ((packet->p_operation == op_response || packet->p_operation == op_response_piggyback) &&
 		!vector[1])
 	{
-		Firebird::Arg::StatusVector s(vector);
+		Arg::StatusVector s(vector);
 		s.copyTo(warning);
 		return;
 	}
@@ -1151,10 +1236,10 @@ void rem_port::checkResponse(Firebird::IStatus* warning, PACKET* packet, bool ch
 
 	if (!vector[1])
 	{
-		Firebird::Arg::Gds(isc_net_read_err).raise();
+		Arg::Gds(isc_net_read_err).raise();
 	}
 
-	Firebird::status_exception::raise(vector);
+	status_exception::raise(vector);
 }
 
 static void setCStr(CSTRING& to, const char* from)
@@ -1166,10 +1251,10 @@ static void setCStr(CSTRING& to, const char* from)
 
 void rem_port::addServerKeys(const CSTRING* passedStr)
 {
-	Firebird::ClumpletReader newKeys(Firebird::ClumpletReader::UnTagged,
+	ClumpletReader newKeys(ClumpletReader::UnTagged,
 									 passedStr->cstr_address, passedStr->cstr_length);
 
-	Firebird::PathName type, plugins, plugin;
+	PathName type, plugins, plugin;
 	unsigned len;
 	KnownServerKey* currentKey = nullptr;
 	for (newKeys.rewind(); !newKeys.isEof(); newKeys.moveNext())
@@ -1228,7 +1313,7 @@ bool rem_port::tryKeyType(const KnownServerKey& srvKey, InternalCryptKey* cryptK
 		return false;
 	}
 
-	if (getPortConfig()->getWireCrypt(Firebird::WC_CLIENT) == Firebird::WIRE_CRYPT_DISABLED)
+	if (getPortConfig()->getWireCrypt(WC_CLIENT) == WIRE_CRYPT_DISABLED)
 	{
 		port_crypt_complete = true;
 		return true;
@@ -1236,24 +1321,24 @@ bool rem_port::tryKeyType(const KnownServerKey& srvKey, InternalCryptKey* cryptK
 
 	// we got correct key's type pair
 	// check what about crypt plugin for it
-	Firebird::ParsedList clientPlugins(getPortConfig()->getPlugins(Firebird::IPluginManager::TYPE_WIRE_CRYPT));
+	ParsedList clientPlugins(getPortConfig()->getPlugins(IPluginManager::TYPE_WIRE_CRYPT));
 	for (unsigned n = 0; n < clientPlugins.getCount(); ++n)
 	{
-		Firebird::PathName p(clientPlugins[n]);
+		PathName p(clientPlugins[n]);
 		WIRECRYPT_DEBUG(fprintf(stderr, "tryKeyType, client plugin %s\n", p.c_str()));
-		if (srvKey.plugins.find(" " + p + " ") != Firebird::PathName::npos)
+		if (srvKey.plugins.find(" " + p + " ") != PathName::npos)
 		{
 			WIRECRYPT_DEBUG(fprintf(stderr, "tryKeyType, server listed plugin %s\n", p.c_str()));
-			Firebird::GetPlugins<Firebird::IWireCryptPlugin>
-				cp(Firebird::IPluginManager::TYPE_WIRE_CRYPT, p.c_str());
+			GetPlugins<IWireCryptPlugin>
+				cp(IPluginManager::TYPE_WIRE_CRYPT, p.c_str());
 			if (cp.hasData())
 			{
 				WIRECRYPT_DEBUG(fprintf(stderr, "tryKeyType, client loaded plugin %s\n", p.c_str()));
-				Firebird::LocalStatus st;
-				Firebird::CheckStatusWrapper statusWrapper(&st);
+				LocalStatus st;
+				CheckStatusWrapper statusWrapper(&st);
 
 				// Pass IV to plugin
-				//const Firebird::UCharBuffer* specificData = srvKey.findSpecificData(p);
+				//const UCharBuffer* specificData = srvKey.findSpecificData(p);
 				auto* specificData = srvKey.findSpecificData(p);
 				if (specificData)
 				{
@@ -1264,9 +1349,9 @@ bool rem_port::tryKeyType(const KnownServerKey& srvKey, InternalCryptKey* cryptK
 
 				// Pass key to plugin
 				cp.plugin()->setKey(&statusWrapper, cryptKey);
-				if (st.getState() & Firebird::IStatus::STATE_ERRORS)
+				if (st.getState() & IStatus::STATE_ERRORS)
 				{
-					Firebird::status_exception::raise(&st);
+					status_exception::raise(&st);
 				}
 
 				// Looks like we've found correct crypt plugin and key for it
@@ -1315,20 +1400,20 @@ const unsigned char* SrvAuthBlock::getData(unsigned int* length)
 	return *length ? dataForPlugin.begin() : NULL;
 }
 
-void SrvAuthBlock::putData(Firebird::CheckStatusWrapper* status, unsigned int length, const void* data)
+void SrvAuthBlock::putData(CheckStatusWrapper* status, unsigned int length, const void* data)
 {
 	status->init();
 	try
 	{
 		memcpy(dataFromPlugin.getBuffer(length), data, length);
 	}
-	catch (const Firebird::Exception& ex)
+	catch (const Exception& ex)
 	{
 		ex.stuffException(status);
 	}
 }
 
-Firebird::ICryptKey* SrvAuthBlock::newKey(Firebird::CheckStatusWrapper* status)
+ICryptKey* SrvAuthBlock::newKey(CheckStatusWrapper* status)
 {
 	status->init();
 	try
@@ -1342,14 +1427,14 @@ Firebird::ICryptKey* SrvAuthBlock::newKey(Firebird::CheckStatusWrapper* status)
 
 		return k;
 	}
-	catch (const Firebird::Exception& ex)
+	catch (const Exception& ex)
 	{
 		ex.stuffException(status);
 	}
 	return NULL;
 }
 
-void rem_port::versionInfo(Firebird::string& version) const
+void rem_port::versionInfo(string& version) const
 {
 	version.printf("%s/%s", FB_VERSION, port_version->str_data);
 #ifndef WIRE_COMPRESS_SUPPORT
@@ -1367,7 +1452,7 @@ void rem_port::versionInfo(Firebird::string& version) const
 
 
 #ifdef WIRE_COMPRESS_SUPPORT
-static Firebird::InitInstance<Firebird::ZLib> zlib;
+static InitInstance<ZLib> zlib;
 #endif // WIRE_COMPRESS_SUPPORT
 
 rem_port::~rem_port()
@@ -1389,7 +1474,7 @@ rem_port::~rem_port()
 	}
 
 	if (port_crypt_plugin)
-		Firebird::PluginManagerInterfacePtr()->releasePlugin(port_crypt_plugin);
+		PluginManagerInterfacePtr()->releasePlugin(port_crypt_plugin);
 
 #ifdef DEV_BUILD
 	--portCounter;
@@ -1409,7 +1494,12 @@ bool REMOTE_inflate(rem_port* port, PacketReceive* packet_receive, UCHAR* buffer
 {
 #ifdef WIRE_COMPRESS_SUPPORT
 	if (!port->port_compressed)
-		return packet_receive(port, buffer, buffer_length, length);
+	{
+		const bool ret = packet_receive(port, buffer, buffer_length, length);
+		if (ret)
+			port->bumpLogBytes(rem_port::RECEIVE, *length);
+		return ret;
+	}
 
 	z_stream& strm = port->port_recv_stream;
 	strm.avail_out = buffer_length;
@@ -1481,16 +1571,22 @@ bool REMOTE_inflate(rem_port* port, PacketReceive* packet_receive, UCHAR* buffer
 	fprintf(stderr, "ZLib buffer %s\n", port->port_z_data ? "has data" : "is empty");
 #endif
 
+	port->bumpLogBytes(rem_port::RECEIVE, *length);
 	return true;
 #else
-	return packet_receive(port, buffer, buffer_length, length);
+	const bool ret = packet_receive(port, buffer, buffer_length, length);
+	if (ret)
+		port->bumpLogBytes(rem_port::RECEIVE, *length);
+	return ret;
 #endif
 }
 
 bool REMOTE_deflate(RemoteXdr* xdrs, ProtoWrite* proto_write, PacketSend* packet_send, bool flush)
 {
-#ifdef WIRE_COMPRESS_SUPPORT
 	rem_port* port = xdrs->x_public;
+	port->bumpLogBytes(rem_port::SEND, xdrs->x_private - xdrs->x_base);
+
+#ifdef WIRE_COMPRESS_SUPPORT
 	if (!(port->port_compressed && (port->port_flags & PORT_compressed)))
 		return proto_write(xdrs);
 
@@ -1575,16 +1671,16 @@ void rem_port::initCompression()
 #ifdef WIRE_COMPRESS_SUPPORT
 	if (port_protocol >= PROTOCOL_VERSION13 && !port_compressed && zlib())
 	{
-		port_send_stream.zalloc = Firebird::ZLib::allocFunc;
-		port_send_stream.zfree = Firebird::ZLib::freeFunc;
+		port_send_stream.zalloc = ZLib::allocFunc;
+		port_send_stream.zfree = ZLib::freeFunc;
 		port_send_stream.opaque = Z_NULL;
 		int ret = zlib().deflateInit(&port_send_stream, Z_DEFAULT_COMPRESSION);
 		if (ret != Z_OK)
-			(Firebird::Arg::Gds(isc_deflate_init) << Firebird::Arg::Num(ret)).raise();
+			(Arg::Gds(isc_deflate_init) << Arg::Num(ret)).raise();
 		port_send_stream.next_out = NULL;
 
-		port_recv_stream.zalloc = Firebird::ZLib::allocFunc;
-		port_recv_stream.zfree = Firebird::ZLib::freeFunc;
+		port_recv_stream.zalloc = ZLib::allocFunc;
+		port_recv_stream.zfree = ZLib::freeFunc;
 		port_recv_stream.opaque = Z_NULL;
 		port_recv_stream.avail_in = 0;
 		port_recv_stream.next_in = Z_NULL;
@@ -1592,14 +1688,14 @@ void rem_port::initCompression()
 		if (ret != Z_OK)
 		{
 			zlib().deflateEnd(&port_send_stream);
-			(Firebird::Arg::Gds(isc_inflate_init) << Firebird::Arg::Num(ret)).raise();
+			(Arg::Gds(isc_inflate_init) << Arg::Num(ret)).raise();
 		}
 
 		try
 		{
 			port_compressed.reset(FB_NEW_POOL(getPool()) UCHAR[port_buff_size * 2]);
 		}
-		catch (const Firebird::Exception&)
+		catch (const Exception&)
 		{
 			zlib().deflateEnd(&port_send_stream);
 			zlib().inflateEnd(&port_recv_stream);
@@ -1617,7 +1713,7 @@ void rem_port::initCompression()
 }
 
 
-void InternalCryptKey::setSymmetric(Firebird::CheckStatusWrapper* status, const char* type,
+void InternalCryptKey::setSymmetric(CheckStatusWrapper* status, const char* type,
 	unsigned keyLength, const void* key)
 {
 	try
@@ -1627,13 +1723,13 @@ void InternalCryptKey::setSymmetric(Firebird::CheckStatusWrapper* status, const
 		encrypt.set(keyLength, key);
 		decrypt.clear();
 	}
-	catch (const Firebird::Exception& ex)
+	catch (const Exception& ex)
 	{
 		ex.stuffException(status);
 	}
 }
 
-void InternalCryptKey::setAsymmetric(Firebird::CheckStatusWrapper* status, const char* type,
+void InternalCryptKey::setAsymmetric(CheckStatusWrapper* status, const char* type,
 	unsigned encryptKeyLength, const void* encryptKey, unsigned decryptKeyLength,
 	const void* decryptKey)
 {
@@ -1644,7 +1740,7 @@ void InternalCryptKey::setAsymmetric(Firebird::CheckStatusWrapper* status, const
 		encrypt.set(encryptKeyLength, encryptKey);
 		decrypt.set(decryptKeyLength, decryptKey);
 	}
-	catch (const Firebird::Exception& ex)
+	catch (const Exception& ex)
 	{
 		ex.stuffException(status);
 	}
