@@ -308,7 +308,7 @@ namespace {
 		public Firebird::AutoIface<Firebird::IListUsersImpl<Callback, Firebird::CheckStatusWrapper> >
 	{
 	public:
-		explicit Callback(StackUserData* pu)
+		explicit Callback(UserData* pu)
 			: u(pu)
 		{ }
 
@@ -329,7 +329,7 @@ namespace {
 		}
 
 	private:
-		StackUserData* u;
+		UserData* u;
 	};
 } // anonymous namespace
 
@@ -356,7 +356,7 @@ int gsec(Firebird::UtilSvc* uSvc)
 	tsec* tdsec = &tsecInstance;
 	tsec::putSpecific(tdsec);
 
-	StackUserData u;
+	UserData u;
 	tdsec->tsec_user_data = &u;
 
 	Firebird::LocalStatus lsManager;
@@ -549,7 +549,7 @@ int gsec(Firebird::UtilSvc* uSvc)
 				if (user_data->operation() == MOD_OPER && user_data->userName()->entered() &&
 					(fieldSet(&user_data->u) || fieldSet(&user_data->g) || fieldSet(&user_data->group)))
 				{
-					StackUserData u;
+					UserData u;
 					u.op = DIS_OPER;
 					u.user.set(&statusWrapper, user_data->userName()->get());
 					check(&statusWrapper);
